# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/ipfscidinputdialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CIDInputDialog(object):
    def setupUi(self, CIDInputDialog):
        CIDInputDialog.setObjectName("CIDInputDialog")
        CIDInputDialog.resize(676, 103)
        self.verticalLayout = QtWidgets.QVBoxLayout(CIDInputDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(CIDInputDialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.cid = QtWidgets.QLineEdit(CIDInputDialog)
        self.cid.setMaxLength(256)
        self.cid.setObjectName("cid")
        self.horizontalLayout.addWidget(self.cid)
        self.clearButton = QtWidgets.QPushButton(CIDInputDialog)
        self.clearButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/clear-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.clearButton.setIcon(icon)
        self.clearButton.setObjectName("clearButton")
        self.horizontalLayout.addWidget(self.clearButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.validity = QtWidgets.QLabel(CIDInputDialog)
        self.validity.setText("")
        self.validity.setAlignment(QtCore.Qt.AlignCenter)
        self.validity.setObjectName("validity")
        self.verticalLayout_2.addWidget(self.validity)
        self.buttonBox = QtWidgets.QDialogButtonBox(CIDInputDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.verticalLayout.addLayout(self.verticalLayout_2)

        self.retranslateUi(CIDInputDialog)
        self.buttonBox.accepted.connect(CIDInputDialog.accept)
        self.buttonBox.rejected.connect(CIDInputDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CIDInputDialog)

    def retranslateUi(self, CIDInputDialog):
        _translate = QtCore.QCoreApplication.translate
        CIDInputDialog.setWindowTitle(_translate("CIDInputDialog", "Dialog"))
        self.label.setText(_translate("CIDInputDialog", "IPFS CID"))
        self.clearButton.setToolTip(_translate("CIDInputDialog", "<html><head/><body><p>Clear</p></body></html>"))
        self.clearButton.setShortcut(_translate("CIDInputDialog", "Ctrl+X"))

from . import galacteek_rc




from PyQt5.QtWidgets import QWidget, QTextEdit, QAction
from PyQt5.QtWidgets import QMessageBox

from PyQt5.QtCore import QUrl, Qt, QTemporaryFile, QTemporaryDir, QSaveFile
from PyQt5.QtCore import QIODevice, pyqtSignal

from . import ui_newdocument
from .helpers import *
from .widgets import *
from .i18n import *
from galacteek.ipfs.ipfsops import *
from galacteek.appsettings import *
from galacteek.ipfs.wrappers import ipfsOp

def iImportedDocument(name, hash):
    return QCoreApplication.translate('NewDocumentForm',
        'Succesfully imported {0} in the Documents folder (hash reference {1})').format(
                name, hash)

def iImportError():
    return QCoreApplication.translate('NewDocumentForm',
        'File import error')

class AddDocumentWidget(GalacteekTab):
    importSuccess = pyqtSignal(str, dict)

    def __init__(self, gWindow, *args, **kw):
        super(AddDocumentWidget, self).__init__(gWindow, *args, **kw)

        self.ui = ui_newdocument.Ui_NewDocumentForm()
        self.ui.setupUi(self)
        self.ui.importButton.clicked.connect(self.onImport)
        self.importSuccess.connect(self.onSuccess)

    @ipfsOp
    async def importFile(self, op):
        wrapEnabled = self.app.settingsMgr.isTrue(
            CFG_SECTION_UI, CFG_KEY_WRAPSINGLEFILES)
        filename = self.ui.filename.text()
        if filename == '':
            return messageBox(iGeneralError('Please specify a filename'))

        text = self.ui.textEdit.toPlainText()
        tempDir = QTemporaryDir()

        if not tempDir.isValid():
            return messageBox(iGeneralError('Error creating directory'))

        tempFilePath = tempDir.filePath(filename)

        tempFile = QSaveFile(tempFilePath)
        tempFile.open(QIODevice.WriteOnly)
        tempFile.write(text.encode('utf-8'))
        tempFile.commit()

        root = await op.addPath(tempFile.fileName(), wrap=wrapEnabled)
        if await op.filesLink(root, self.profile.pathDocuments, name=filename):
            self.importSuccess.emit(filename, root)
        else:
            messageBox(iImportError())

    def onSuccess(self, filename, entry):
        messageBox(iImportedDocument(filename, entry['Hash']))
        self.gWindow.removeTabFromWidget(self)

    def onImport(self):
        self.app.task(self.importFile)




import collections
import os.path
import json
import time
import sys
import collections
import copy
from datetime import datetime

from async_generator import async_generator, yield_, yield_from_
import asyncio
import aiofiles

from galacteek.core.asynclib import asyncify

from PyQt5.QtCore import pyqtSignal, QUrl, QObject

class MarksEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, IPFSHashMark):
            return obj.data
        return json.JSONEncoder.default(self, obj)

marksKey = '_marks'

def rSlash(path):
    return path.rstrip('/')

class IPFSHashMark(collections.UserDict):
    @property
    def markData(self):
        return self.data[self.path]

    def addTags(self, tags):
        self.data[self.path]['tags'] += tags

    def dump(self):
        print(json.dumps(self.data, indent=4))

    @staticmethod
    def fromJson(data):
        return IPFSHashMark(data)

    @staticmethod
    def make(path, title=None, datecreated=None, share=False, tags=[],
            description='', comment='', datasize=None, cumulativesize=None,
            numlinks=None):
        if datecreated is None:
            datecreated = datetime.now().isoformat()

        path = rSlash(path)

        mData = IPFSHashMark({
            path: {
                'metadata': {
                    'title': title,
                    'description': description,
                    'datasize': datasize,
                    'cumulativesize': cumulativesize,
                    'numlinks': numlinks,
                },
                'datecreated': datecreated,
                'tscreated': int(time.time()),
                'comment': comment,
                'share': share,
                'tags': tags,
            }
        })

        mData.path = path
        return mData

class IPFSMarks(QObject):
    changed = pyqtSignal()
    markDeleted = pyqtSignal(str)
    markAdded = pyqtSignal(str, dict)

    def __init__(self, path, parent=None, autosave=True):
        super().__init__(parent)

        self._path = path
        self._autosave = autosave
        self._marks = self.load()
        self.changed.connect(self.onChanged)
        self.lastsaved = time.time()
        self.changed.emit()

    @property
    def path(self):
        return self._path

    @property
    def autosave(self):
        return self._autosave

    @property
    def asyncQ(self):
        """ Async query """
        return _AsyncMarksQuery(self)

    @property
    def _root(self):
        return self._marks

    @property
    def _rootMarks(self):
        return self._root['ipfsmarks']

    @property
    def _rootCategories(self):
        return self._rootMarks['categories']

    @property
    def _rootFeeds(self):
        return self._root['feeds']

    def load(self):
        try:
            marks = json.load(open(self.path, 'rt'))
            return marks
        except Exception as e:
            marks = collections.OrderedDict()

        if not 'ipfsmarks' in marks:
            marks['ipfsmarks'] = {}
            marks['ipfsmarks']['categories'] = {}
        if not 'feeds' in marks:
            marks['feeds'] = {}

        return marks

    def onChanged(self):
        if self.autosave is True:
            self.save()

    def save(self):
        """ Save synchronously """
        with open(self.path, 'w+t') as fd:
            self.serialize(fd)
            self.lastsaved = time.time()

    async def saveAsync(self):
        async with aiofiles.open(self.path, 'w+t') as fd:
            await fd.write(json.dumps(self._root, indent=4, cls=MarksEncoder))
            self.lastsaved = time.time()

    def hasCategory(self, category, parent=None):
        if parent is None:
            parent = self._rootCategories
        return category in parent

    def enterCategory(self, section, create=False):
        comps = section.lstrip('/').rstrip('/').split('/')
        return self.walk(comps, create=create)

    def walk(self, path, create=True):
        # Walk to a category and create intermediary parents if create
        # is True. path is a list of category components 
        # e.g ['general', 'news'] for category path /general/news
        def _walk(path, parent=None):
            for p in path:
                if p.startswith('_'):
                    return
                if p in parent.keys():
                    parent = parent[p]
                elif p not in parent.keys() and create is True:
                    self.addCategory(p, parent=parent)
                    parent = parent[p]
                else:
                    return
            return parent
        return _walk(path, parent=self._rootCategories)

    def getCategories(self):
        def _list(path, parent=None):
            for p in parent.keys():
                if p.startswith('_'):
                    continue

                fullPath = path + [p]
                yield '/'.join(fullPath)
                yield from  _list(fullPath, parent=parent[p])

        return list(_list([], parent=self._rootCategories))

    def addCategory(self, category, parent=None):
        if parent is None:
            parent = self._rootCategories

        if len(category) > 256:
            return None

        if not self.hasCategory(category, parent=parent):
            parent[category] = {
                marksKey: {}
            }
            self.changed.emit()
            return parent[category]

    def getCategoryMarks(self, category, usecopy=False):
        sec = self.enterCategory(category)
        if sec:
            if usecopy:
                return copy.copy(sec[marksKey])
            else:
                return sec[marksKey]

    def getAll(self, share=False):
        cats = self.getCategories()
        _all = {}
        for cat in cats:
            catMarks = self.getCategoryMarks(cat)
            for mpath, mark in catMarks.items():
                if mark['share'] == share:
                    _all[mpath] = mark
        return _all

    def search(self, bpath, category=None, tags=[], delete=False):
        path = rSlash(bpath)
        categories = self.getCategories()

        for cat in categories:
            if category and cat != category:
                continue

            marks = self.getCategoryMarks(cat)
            if not marks:
                continue

            if path in marks.keys():
                if delete is True:
                    del marks[path]
                    self.markDeleted.emit(path)
                    self.changed.emit()
                    return True

                tagsOk = True
                m = marks[path]
                for stag in tags:
                    if stag not in m['tags']:
                        tagsOk = False

                if tagsOk:
                    return (path, marks[path])

    def insertMark(self, mark, category):
        # Insert a mark in given category, checking of already existing mark
        # is left to the caller
        sec = self.enterCategory(category, create=True)
        if not sec:
            return False

        # Handle IPFSHashMark or tuple
        if isinstance(mark, IPFSHashMark):
            sec[marksKey].update(mark)
            self.markAdded.emit(mark.path, mark.markData)
        else:
            try:
                mPath, mData = mark
                sec[marksKey][mPath] = mData
                self.markAdded.emit(mPath, mData)
            except Exception as e:
                return False

        self.changed.emit()
        return True

    def add(self, bpath, title=None, category='general', share=False, tags=[]):
        if not bpath:
            return None

        path = rSlash(bpath)

        sec = self.enterCategory(category, create=True)

        if not sec:
            return False

        if self.search(path):
            # We already have stored a mark with this path
            return False

        mark = IPFSHashMark.make(path,
            title=title,
            datecreated=datetime.now().isoformat(),
            share=share,
            tags=tags,
        )

        sec[marksKey].update(mark)
        self.changed.emit()
        self.markAdded.emit(path, mark.markData)

        return True

    def delete(self, path):
        return self.search(path, delete=True)

    def merge(self, oMarks):
        for cat in oMarks.getCategories():
            marks = oMarks.getCategoryMarks(cat)
            for mark in marks.items():
                r = self.insertMark(mark, cat)

    def follow(self, ipnsp, name, active=True, maxentries=4096,
            resolveevery=3600, share=False, autoPin=False):
        if ipnsp is None:
            return

        feedsSec = self._rootFeeds
        ipnsp = rSlash(ipnsp)

        if ipnsp in feedsSec:
            return
        feedsSec[ipnsp] = {
            'name': name,
            'active': active,
            'maxentries': maxentries,
            'resolvepolicy': 'auto',
            'resolveevery': resolveevery,
            'resolvedlast': None,
            'share': share,
            'autopin': autoPin,
            marksKey: {},
        }
        self.changed.emit()
        return feedsSec[ipnsp]

    def feedAddMark(self, ipnsp, mark):
        feeds = self._rootFeeds
        if not ipnsp in feeds:
            return False
        sec = feeds[ipnsp][marksKey]
        if mark.path in sec:
            return False

        feeds[ipnsp][marksKey].update(mark)
        self.changed.emit()
        return True

    def getFeeds(self):
        return list(self._rootFeeds.items())

    def getFeedMarks(self, path):
        feeds = self.getFeeds()
        for fPath, fData in feeds:
            if fPath == path:
                return fData[marksKey]

    def serialize(self, fd):
        return json.dump(self._root, fd, indent=4, cls=MarksEncoder)

    def dump(self):
        print(self.serialize(sys.stdout))

    def norm(self, path):
        return path.rstrip('/')

class _AsyncMarksQuery:
    """
    Class designed to make operations on the marks tree asynchronously.
    """
    def __init__(self, marks):
        self.m = marks

    async def search(self, bpath, category=None, tags=[], delete=False):
        path = rSlash(bpath)

        getCategories = await self.getCategories()

        for cat in getCategories:
            await asyncio.sleep(0)
            if category and cat != category:
                continue

            marks = await self.getCategoryMarks(cat)
            if path in marks:
                if delete is True:
                    del marks[path]
                    self.changed.emit()
                    return True

                tagsOk = True
                m = marks[path]
                for stag in tags:
                    if stag not in m['tags']:
                        tagsOk = False

                if tagsOk:
                    return marks[path]

    async def add(self, bpath, title=None, category='general', share=False,
            tags=[]):
        if not bpath:
            return None

        path = rSlash(bpath)
        sec =  self.m.enterCategory(category, create=True)
        if not sec:
            return False

        if await self.search(path):
            # We have already stored a mark with this path
            return False

        mark = IPFSHashMark.make(path,
            title=title,
            datecreated=datetime.now().isoformat(),
            share=share,
            tags=tags,
        )

        sec[marksKey].update(mark)
        self.m.changed.emit()

        return True

    async def walkAsync(self, path, create=True):
        async def _w(path, parent=None):
            for p in path:
                await asyncio.sleep(0)
                if p.startswith('_'):
                    return
                if p in parent.keys():
                    parent = parent[p]
                elif p not in parent.keys() and create is True:
                    self.addCategory(p, parent=parent)
                    parent = parent[p]
                else:
                    return
            return parent

        return await _w(path, parent=self.m._rootCategories)

    async def getCategories(self):
        @async_generator
        async def _list(path, parent=None):
            for p in parent.keys():
                await asyncio.sleep(0)
                if p.startswith('_'):
                    continue

                fullPath = path + [p]
                await yield_( '/'.join(fullPath) )
                await yield_from_( _list(path + [p], parent=parent[p]) )

        cats = []
        async for v in _list([], parent=self.m._rootCategories):
            cats.append(v)
        return cats

    async def getCategoryMarks(self, category):
        sec = self.m.enterCategory(category)
        if sec:
            return copy.copy(sec[marksKey])

    async def enterCategory(self, section, create=False):
        comps = section.lstrip('/').rstrip('/').split('/')
        return await self.walkAsync(comps, create=create)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/browsertab.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_BrowserTabForm(object):
    def setupUi(self, BrowserTabForm):
        BrowserTabForm.setObjectName("BrowserTabForm")
        BrowserTabForm.resize(811, 444)
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(BrowserTabForm)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setContentsMargins(-1, -1, -1, 10)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.loadIpfsButton = QtWidgets.QToolButton(BrowserTabForm)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-logo-128-ice.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadIpfsButton.setIcon(icon)
        self.loadIpfsButton.setObjectName("loadIpfsButton")
        self.horizontalLayout.addWidget(self.loadIpfsButton)
        self.loadFromClipboardButton = QtWidgets.QPushButton(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loadFromClipboardButton.sizePolicy().hasHeightForWidth())
        self.loadFromClipboardButton.setSizePolicy(sizePolicy)
        self.loadFromClipboardButton.setMaximumSize(QtCore.QSize(40, 24))
        self.loadFromClipboardButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/share/icons/clipboard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadFromClipboardButton.setIcon(icon1)
        self.loadFromClipboardButton.setObjectName("loadFromClipboardButton")
        self.horizontalLayout.addWidget(self.loadFromClipboardButton)
        self.urlZone = QtWidgets.QLineEdit(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.urlZone.sizePolicy().hasHeightForWidth())
        self.urlZone.setSizePolicy(sizePolicy)
        self.urlZone.setMinimumSize(QtCore.QSize(400, 0))
        self.urlZone.setMaximumSize(QtCore.QSize(1000, 26))
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        font.setPointSize(11)
        font.setItalic(False)
        self.urlZone.setFont(font)
        self.urlZone.setStyleSheet("background-color: rgb(189, 236, 255);")
        self.urlZone.setMaxLength(2048)
        self.urlZone.setObjectName("urlZone")
        self.horizontalLayout.addWidget(self.urlZone)
        self.backButton = QtWidgets.QToolButton(BrowserTabForm)
        self.backButton.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/share/icons/go-previous.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.backButton.setIcon(icon2)
        self.backButton.setObjectName("backButton")
        self.horizontalLayout.addWidget(self.backButton)
        self.forwardButton = QtWidgets.QToolButton(BrowserTabForm)
        self.forwardButton.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/share/icons/go-next.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.forwardButton.setIcon(icon3)
        self.forwardButton.setObjectName("forwardButton")
        self.horizontalLayout.addWidget(self.forwardButton)
        self.stopButton = QtWidgets.QPushButton(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.stopButton.sizePolicy().hasHeightForWidth())
        self.stopButton.setSizePolicy(sizePolicy)
        self.stopButton.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/share/icons/cancel.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stopButton.setIcon(icon4)
        self.stopButton.setObjectName("stopButton")
        self.horizontalLayout.addWidget(self.stopButton)
        self.refreshButton = QtWidgets.QPushButton(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        self.refreshButton.setMaximumSize(QtCore.QSize(50, 24))
        self.refreshButton.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/share/icons/refresh.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon5)
        self.refreshButton.setObjectName("refreshButton")
        self.horizontalLayout.addWidget(self.refreshButton)
        self.pinAllButton = QtWidgets.QToolButton(BrowserTabForm)
        self.pinAllButton.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/share/icons/pin-black.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pinAllButton.setIcon(icon6)
        self.pinAllButton.setObjectName("pinAllButton")
        self.horizontalLayout.addWidget(self.pinAllButton)
        self.actionComboBox = QtWidgets.QComboBox(BrowserTabForm)
        self.actionComboBox.setObjectName("actionComboBox")
        self.horizontalLayout.addWidget(self.actionComboBox)
        self.hashmarkPageButton = QtWidgets.QPushButton(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hashmarkPageButton.sizePolicy().hasHeightForWidth())
        self.hashmarkPageButton.setSizePolicy(sizePolicy)
        self.hashmarkPageButton.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/share/icons/hashmarks.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.hashmarkPageButton.setIcon(icon7)
        self.hashmarkPageButton.setObjectName("hashmarkPageButton")
        self.horizontalLayout.addWidget(self.hashmarkPageButton)
        self.progressBar = QtWidgets.QProgressBar(BrowserTabForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progressBar.sizePolicy().hasHeightForWidth())
        self.progressBar.setSizePolicy(sizePolicy)
        self.progressBar.setMinimumSize(QtCore.QSize(40, 0))
        self.progressBar.setMaximumSize(QtCore.QSize(30, 16777215))
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.horizontalLayout.addWidget(self.progressBar)
        self.horizontalLayout_7.addLayout(self.horizontalLayout)
        self.verticalLayout_17.addLayout(self.horizontalLayout_7)
        self.vLayoutBrowser = QtWidgets.QVBoxLayout()
        self.vLayoutBrowser.setObjectName("vLayoutBrowser")
        self.verticalLayout_17.addLayout(self.vLayoutBrowser)
        self.verticalLayout_18.addLayout(self.verticalLayout_17)

        self.retranslateUi(BrowserTabForm)
        QtCore.QMetaObject.connectSlotsByName(BrowserTabForm)

    def retranslateUi(self, BrowserTabForm):
        _translate = QtCore.QCoreApplication.translate
        BrowserTabForm.setWindowTitle(_translate("BrowserTabForm", "Form"))
        self.loadIpfsButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Browse IPFS</p></body></html>"))
        self.loadIpfsButton.setText(_translate("BrowserTabForm", "..."))
        self.loadFromClipboardButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Load IPFS CID from clipboard</p></body></html>"))
        self.backButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Previous</p></body></html>"))
        self.forwardButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Next</p></body></html>"))
        self.refreshButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Reload page</p></body></html>"))
        self.pinAllButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Automatically pins visited pages</p></body></html>"))
        self.actionComboBox.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Pin</p></body></html>"))
        self.hashmarkPageButton.setToolTip(_translate("BrowserTabForm", "<html><head/><body><p>Hashmark this page</p></body></html>"))

from . import galacteek_rc




from PyQt5.QtCore import QFileSystemWatcher, QObject, pyqtSignal

class FileWatcher(QObject):
    pathChanged = pyqtSignal(str)

    def __init__(self):
        super().__init__()

        self._watcher = QFileSystemWatcher(self)
        self._watcher.directoryChanged.connect(self.onChanged)
        self._watcher.fileChanged.connect(self.onChanged)
        self._events = {}

    @property
    def watched(self):
        return self._watcher.directories() + self._watcher.files()

    def watch(self, path):
        self._watcher.addPath(path)

    def onChanged(self, path):
        self.pathChanged.emit(path)



import asyncio
import functools
import jinja2, jinja2.exceptions

from galacteek import log
from galacteek.ipfs.wrappers import ipfsOp, ipfsOpFn

def defaultEnv():
    return jinja2.Environment(
	loader=jinja2.PackageLoader('galacteek', 'templates'),
	autoescape=jinja2.select_autoescape(['html', 'xml']))

@ipfsOpFn
async def ipfsRender(op, tmplname, **kw):
    env = defaultEnv()
    loop = kw.pop('loop', None)

    tmpl = env.get_template(tmplname)
    if not tmpl:
        raise Exception('template not found')

    try:
        loop = loop if loop else asyncio.get_event_loop()
        data = await loop.run_in_executor(None, functools.partial(tmpl.render, **kw))
        ent = await op.client.add_str(data)
    except Exception as e:
        log.debug('Could not render web template {0}'.format(tmplname))
        return None
    else:
        return ent['Hash']



from jsontraverse.parser import JsonTraverseParser

class CTraverseParser(JsonTraverseParser):
    """
    Custom JsonTraverseParser (from json-traverse) that works directly on a
    preloaded object
    """

    def __init__(self, data):
        self.data = data

def traverseParser(data):
    return CTraverseParser(data)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/newdocument.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_NewDocumentForm(object):
    def setupUi(self, NewDocumentForm):
        NewDocumentForm.setObjectName("NewDocumentForm")
        NewDocumentForm.resize(597, 335)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(NewDocumentForm)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.textEdit = QtWidgets.QTextEdit(NewDocumentForm)
        self.textEdit.setObjectName("textEdit")
        self.verticalLayout.addWidget(self.textEdit)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(NewDocumentForm)
        self.label.setMaximumSize(QtCore.QSize(150, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.filename = QtWidgets.QLineEdit(NewDocumentForm)
        self.filename.setMaximumSize(QtCore.QSize(1024, 16777215))
        self.filename.setMaxLength(256)
        self.filename.setObjectName("filename")
        self.horizontalLayout.addWidget(self.filename)
        self.importButton = QtWidgets.QPushButton(NewDocumentForm)
        self.importButton.setObjectName("importButton")
        self.horizontalLayout.addWidget(self.importButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(NewDocumentForm)
        QtCore.QMetaObject.connectSlotsByName(NewDocumentForm)

    def retranslateUi(self, NewDocumentForm):
        _translate = QtCore.QCoreApplication.translate
        NewDocumentForm.setWindowTitle(_translate("NewDocumentForm", "Form"))
        self.label.setText(_translate("NewDocumentForm", "File name"))
        self.importButton.setToolTip(_translate("NewDocumentForm", "<html><head/><body><p>Import this document to IPFS</p></body></html>"))
        self.importButton.setText(_translate("NewDocumentForm", "Import"))





import pytest

import tempfile
import time
import os
import json

import asyncio
import aioipfs

from PyQt5.QtCore import QObject, pyqtSignal

from galacteek.ipfs import asyncipfsd
from galacteek.ipfs import ipfsdconfig
from galacteek.core.ipfsmarks import *
from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.tunnel import *
from galacteek.ipfs.pubsub import *

from .daemon import *

class EchoProtocol(asyncio.Protocol):
    def __init__(self, exitF):
        super().__init__()
        self.exitF = exitF
        self.msgCount = 0

    def connection_made(self, transport):
        print('Connection made')
        self.transport = transport
        self.transport.write('Hello'.encode())

    def data_received(self, data):
        self.msgCount += 1
        if self.msgCount < 5:
            self.transport.write(data)
        else:
            self.transport.close()

    def eof_received(self):
        pass

    def connection_lost(self, exc):
        self.exitF.set_result(True)

class TestIPFSD:
    @pytest.mark.asyncio
    async def test_basic(self, event_loop, ipfsdaemon, iclient, ipfsop):
        async def tests(op):
            id = await op.client.core.id()

            slashList = await op.filesList('/')
            r = await op.client.add_json({'a': 123})
            assert await op.filesLink(r, '/') == True

            await op.client.close()

        def cbstarted(f):
            event_loop.create_task(tests(ipfsop))

        started = await ipfsdaemon.start()
        assert started == True

        await ipfsdaemon.proto.eventStarted.wait()
        await tests(ipfsop)

        ipfsdaemon.stop()
        await asyncio.wait([ipfsdaemon.exitFuture])

    @pytest.mark.asyncio
    async def test_tunnel(self, event_loop, ipfsdaemon, ipfsdaemon2,
            iclient, iclient2, ipfsop):
        # Connect two nodes, create a P2P listener on the first one and make
        # the second node do the stream dial.

        protoF = asyncio.Future()
        proto = EchoProtocol(protoF)
        port = 10010
        protoName = 'test'
        p2pL = P2PListener(iclient, protoName, ('127.0.0.1', port),
                lambda: proto)

        async def tcpEchoClient(portdest, message, loop):
            reader, writer = await asyncio.open_connection('127.0.0.1',
                    portdest)
            p = EchoProtocol(asyncio.Future())

            await asyncio.sleep(2)

            encoded = message.encode()
            writer.write(encoded)
            data = await reader.read(100)
            print('Received: %r' % data.decode())
            assert data == encoded
            writer.close()

        async def createTunnel(op):
            id1 = await iclient.id()
            id2 = await iclient2.id()

            # Swarm connect the two nodes
            await iclient2.swarm.connect(
                '/ip4/127.0.0.1/tcp/{0}/ipfs/{1}'.format(
                    ipfsdaemon.swarmport, id1['ID']))

            peers2 = await iclient2.swarm.peers()
            listener = await p2pL.open()
            await iclient.p2p.listener_ls(headers=True)

            assert listener['Protocol'] == '/p2p/{}'.format(protoName)
            assert listener['Address'] == '/ip4/127.0.0.1/tcp/{}'.format(port)

            listenPort = port+1
            r = await iclient2.p2p.stream_dial(id1['ID'], 'test',
                    address='/ip4/127.0.0.1/tcp/{}'.format(listenPort))
            addr = r['Address']

            await tcpEchoClient(listenPort, 'Hello', event_loop)
            streams = await iclient2.p2p.stream_ls(headers=True)

            assert len(streams['Streams']) > 0
            stream = streams['Streams'].pop()

            assert stream['Protocol'] == '/p2p/{}'.format(protoName)
            assert stream['LocalPeer'] == id2['ID']
            assert stream['LocalAddress'] == '/ip4/127.0.0.1/tcp/{}'.format(
                    listenPort)
            assert stream['RemotePeer'] == id1['ID']

        await startDaemons(event_loop, ipfsdaemon, ipfsdaemon2)
        await createTunnel(ipfsop)

        await asyncio.wait([protoF])
        await p2pL.close()

        stopDaemons(ipfsdaemon, ipfsdaemon2)

        await asyncio.wait([ipfsdaemon.exitFuture, ipfsdaemon2.exitFuture])

        await iclient.close()
        await iclient2.close()

    @pytest.mark.parametrize('hashmarkpath',
            ['/ipfs/QmT1TPVjdZ9CRnqwyQ9WygDoRgRRibFrEyWufenu92SuUV'])
    @pytest.mark.asyncio
    async def test_hashmarks_exchange(self, event_loop, ipfsdaemon, ipfsdaemon2,
            iclient, iclient2, ipfsop, tmpdir, hashmarkpath):
        ipfsdaemon.pubsubEnable = True
        ipfsdaemon2.pubsubEnable = True

        async def exchange():
            marks = IPFSMarks(str(tmpdir.join('marks1.json')))
            marksNet = IPFSMarks(str(tmpdir.join('marks1net.json')))
            marks2 = IPFSMarks(str(tmpdir.join('marks2.json')))
            marks2Net = IPFSMarks(str(tmpdir.join('marks2net.json')))

            m1 = IPFSHashMark.make(hashmarkpath, share=True,
                    description='a shared hashmark',
                    title='stuff')

            def rx1(m):
                print('RX1 >', m)
            def rx2(m):
                print('RX2 >>', m)

            def marksReceived(count):
                assert count == 1
                searchR = marks2Net.search(hashmarkpath)
                assert  searchR[0] == hashmarkpath

                exchanger.stop()
                exchanger2.stop()

                stopDaemons(ipfsdaemon, ipfsdaemon2)

            ctx = Ctx()
            ctx.loop = event_loop
            exchanger = PSHashmarksExchanger(ctx, iclient, marks, marksNet)
            exchanger.start()
            ctx.pubsubMessageRx.connect(rx1)

            ctx2 = Ctx()
            ctx2.loop = event_loop
            ctx2.pubsubMessageRx.connect(rx2)
            ctx2.pubsubMarksReceived.connect(marksReceived)
            exchanger2 = PSHashmarksExchanger(ctx2, iclient2, marks2, marks2Net)
            exchanger2.start()
            await asyncio.sleep(1)
            marks.insertMark(m1, 'shared/hashmarks')

        await startDaemons(event_loop, ipfsdaemon, ipfsdaemon2)
        await exchange()
        await asyncio.wait([ipfsdaemon.exitFuture, ipfsdaemon2.exitFuture])

        await iclient.close()
        await iclient2.close()

class Ctx(QObject):
    pubsubMessageRx = pyqtSignal(dict)
    pubsubMessageTx = pyqtSignal()
    pubsubMarksReceived = pyqtSignal(int)

class Exchanger(PubsubService):
    def __init__(self, client, ipfsCtx):
        super().__init__(client, ipfsCtx, topic='test')

@pytest.fixture()
def configD(tmpdir):
    return ipfsdconfig.getDefault()

class TestConfig:
    def test_default(self, configD):
        cfgStr = str(configD)
        assert 'API' in configD.c
        assert 'Bootstrap' in configD.c




import sys
import asyncio
from datetime import datetime

from PyQt5.QtWidgets import QWidget, QTreeView, QMenu, QHeaderView, QAction
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import QCoreApplication, QUrl, Qt, QObject, QDateTime

from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.cidhelpers import *

from . import ui_hashmarksmgr, ui_hashmarksmgrnetwork, ui_hashmarksmgrfeeds
from .modelhelpers import *
from .helpers import *
from .dialogs import *
from .widgets import *
from .i18n import *

def iPath():
    return QCoreApplication.translate('HashmarksViewForm', 'Path')

def iTitle():
    return QCoreApplication.translate('HashmarksViewForm', 'Title')

def iShared():
    return QCoreApplication.translate('HashmarksViewForm', 'Shared')

def iDate():
    return QCoreApplication.translate('HashmarksViewForm', 'Date')

def iTimestamp():
    return QCoreApplication.translate('HashmarksViewForm', 'Timestamp')

def iAlreadyHashmarked():
    return QCoreApplication.translate('HashmarksViewForm',
        'Already Hashmarked')

def iImportHashmark():
    return QCoreApplication.translate('HashmarksViewForm',
        'Import hashmark to')

def iNetworkMarks():
    return QCoreApplication.translate('HashmarksViewForm', 'Network hashmarks')

def iFeeds():
    return QCoreApplication.translate('HashmarksViewForm', 'Feeds')

class HashmarksView(QTreeView): pass

class HashmarksModel(QStandardItemModel):
    pass

class FeedsModel(QStandardItemModel):
    pass

class CategoryItem(UneditableItem):
    pass

def addHashmark(hashmarks, path, title, description='', stats={}):
    if hashmarks.search(path):
        return messageBox(iAlreadyHashmarked())

    runDialog(AddHashmarkDialog, hashmarks, path, title, description, stats)

class _MarksUpdater:
    def __init__(self):
        self.updatingMarks = False
        self._marksCache = []

    async def updateMarks(self, model, marks, tree, parent=None):
        if self.updatingMarks == True:
            return

        self.updatingMarks = True

        if parent is None:
            parent = model.invisibleRootItem()

        abQuery = marks.asyncQ
        categories = await abQuery.getCategories()

        for cat in categories:
            catItem = None
            ret = await modelSearchAsync(model,
                    parent=parent.index(),
                    maxdepth=1,
                    search=cat, columns=[0])
            if len(ret) == 0:
                catItem = CategoryItem(cat)
                parent.appendRow([catItem])
            else:
                catItem = model.itemFromIndex(ret[0])

            catItemIdx = model.indexFromItem(catItem)

            marks = await abQuery.getCategoryMarks(cat)
            for path, bm in marks.items():
                if path in self._marksCache:
                    continue

                bmTitle = bm['metadata']['title']

                if bmTitle:
                    title = (bmTitle[:64] + '..') if len(bmTitle) > 64 else bmTitle
                    titleTooltip = bmTitle
                else:
                    title = iUnknown()
                    titleTooltip = iUnknown()

                item1 = UneditableItem(title)
                item1.setToolTip(titleTooltip)
                item2 = UneditableItem(path)
                item2.setToolTip(path)
                #item3 = UneditableItem(iYes() if bm['share'] is True else iNo())
                dt = QDateTime.fromString(bm['datecreated'], Qt.ISODate)
                item4 = UneditableItem(dt.toString())
                item5 = UneditableItem(str(bm['tscreated']))
                catItem.appendRow([item1, item2, item4, item5])

                self._marksCache.append(path)

        self.updatingMarks = False
        tree.sortByColumn(0, Qt.AscendingOrder)
        tree.setSortingEnabled(True)
        tree.setModel(model)
        tree.setAlternatingRowColors(True)
        tree.setColumnWidth(0, tree.width() / 3)
        tree.setColumnWidth(1, tree.width() / 3)

class FeedsView(QWidget):
    def __init__(self, marksTab, marks, loop, parent=None):
        super().__init__(parent)

        self.marksTab = marksTab
        self.loop = loop
        self.marks = marks

        self.ui = ui_hashmarksmgrfeeds.Ui_FeedsViewForm()
        self.ui.setupUi(self)

        self.marks.changed.connect(self.updateFeeds)
        self.tree = self.ui.treeFeeds
        self.model = FeedsModel()
        self.model.setHorizontalHeaderLabels([iPath(), iTitle(), iDate()])
        self.tree.setModel(self.model)
        self.tree.doubleClicked.connect(self.onFeedDoubleClick)
        self.tree.setSortingEnabled(True)
        self.tree.header().setSectionResizeMode(0, QHeaderView.ResizeToContents)
        self.tree.setAlternatingRowColors(True)

        self.updateFeeds()

    def updateFeeds(self):
        parent = self.model.invisibleRootItem()
        feeds = self.marks.getFeeds()

        for fPath, fData in feeds:
            fItem = None
            ret = modelSearch(self.model, parent=parent.index(),
                    search=fPath, columns=[0])
            if not ret:
                feedName = fData['name']
                if not feedName:
                    feedName = fPath
                fItem = UneditableItem(feedName)
                parent.appendRow([fItem])
            else:
                fItem = self.model.itemFromIndex(ret[0])

            fItemIdx = self.model.indexFromItem(fItem)
            self.ui.treeFeeds.expand(fItemIdx)

            marks = self.marks.getFeedMarks(fPath)
            for mPath, mData in marks.items():
                ret = modelSearch(self.model, parent=parent.index(),
                        search=mPath, columns=[0])
                if ret: continue

                item1 = UneditableItem(mPath)
                dt = QDateTime.fromString(mData['datecreated'],
                        Qt.ISODate)
                fItem.appendRow([item1,
                    UneditableItem(mData['metadata']['title']),
                    UneditableItem(dt.toString())
                ])

        self.tree.sortByColumn(2, Qt.DescendingOrder)
        self.tree.setSortingEnabled(True)

    def onFeedDoubleClick(self, index):
        indexPath = self.model.sibling(index.row(), 0, index)
        path = self.model.data(indexPath)
        if path:
            self.marksTab.gWindow.addBrowserTab().browseFsPath(path)

class NetworkMarksView(QWidget, _MarksUpdater):
    def __init__(self, marksTab, marks, marksLocal, loop, parent=None):
        super().__init__(parent)

        self.marksTab = marksTab
        self.loop = loop
        self.marks = marks
        self.marksLocal = marksLocal

        self.ui = ui_hashmarksmgrnetwork.Ui_NetworkHashmarksViewForm()
        self.ui.setupUi(self)
        self.ui.search.returnPressed.connect(self.onSearch)
        self.ui.searchButton.clicked.connect(self.onSearch)

        self.model = HashmarksModel()
        self.model.setHorizontalHeaderLabels([iTitle(), iPath(), iDate()])

        self.tree = self.ui.treeNetMarks
        self.tree.setModel(self.model)
        self.tree.doubleClicked.connect(self.onDoubleClick)
        self.tree.header().setSectionResizeMode(0,
                QHeaderView.ResizeToContents)
        self.tree.setContextMenuPolicy(Qt.CustomContextMenu)
        self.tree.customContextMenuRequested.connect(self.onContextMenu)

        self.ui.expandButton.clicked.connect(lambda:
            self.tree.expandAll() if self.ui.expandButton.isChecked()
                else self.tree.collapseAll())

        self.marks.changed.connect(self.doMarksUpdate)

        self.doMarksUpdate()

    def onContextMenu(self, point):
        idx = self.tree.indexAt(point)
        if not idx.isValid():
            return

        idxPath = self.model.sibling(idx.row(), 1, idx)
        dataPath = self.model.data(idxPath)

        localCategories = self.marksLocal.getCategories()
        menu = QMenu()
        catsMenu = QMenu(iImportHashmark())

        def importMark(action):
            cat = action.data()['category']
            mark = action.data()['mark']

            r = self.marksLocal.insertMark(mark, cat)

        mark = self.marks.search(dataPath)
        for cat in localCategories:
            action = QAction(cat, self)
            action.setData({
                'category': cat,
                'mark': mark
            })
            catsMenu.addAction(action)
        catsMenu.triggered.connect(importMark)

        menu.addMenu(catsMenu)
        menu.exec(self.tree.mapToGlobal(point))

    def doMarksUpdate(self):
        self.loop.create_task(self.updateMarks(
            self.model, self.marks, self.tree))

    def onSearch(self):
        text = self.ui.search.text()
        ret = modelSearch(self.model, searchre=text, columns=[0,1])
        if len(ret) > 0:
            idx = ret.pop()
            self.tree.scrollTo(idx)
            self.tree.setCurrentIndex(idx)

    def onDoubleClick(self, index):
        indexPath = self.model.sibling(index.row(), 1, index)
        path = self.model.data(indexPath)
        if path:
            self.marksTab.gWindow.addBrowserTab().browseFsPath(path)

class HashmarksTab(GalacteekTab, _MarksUpdater):
    def __init__(self, *args, **kw):
        super(HashmarksTab, self).__init__(*args, **kw)

        model = kw.pop('model', None)

        self.marksLocal = self.app.marksLocal
        self.marksNetwork = self.app.marksNetwork

        self.marksLocal.changed.connect(self.doMarksUpdate)
        self.marksLocal.markDeleted.connect(self.onMarkDeleted)

        self.ui = ui_hashmarksmgr.Ui_HashmarksViewForm()
        self.ui.setupUi(self)

        self.uiFeeds = FeedsView(self, self.marksLocal,
                self.loop, parent=self)
        self.uiNet = NetworkMarksView(self, self.marksNetwork,
                self.marksLocal, self.loop, parent=self)

        self.ui.toolbox.addItem(self.uiFeeds, iFeeds())
        self.ui.toolbox.addItem(self.uiNet, iNetworkMarks())
        self.ui.expandButton.clicked.connect(lambda:
            self.ui.treeMarks.expandAll() if self.ui.expandButton.isChecked()
                else self.ui.treeMarks.collapseAll())

        self.filter = BasicKeyFilter()
        self.filter.deletePressed.connect(self.onDeletePressed)
        self.installEventFilter(self.filter)

        icon1 = getIcon('hashmarks.png')
        self.ui.toolbox.setItemIcon(0, icon1)
        self.ui.toolbox.setItemIcon(1, icon1)
        self.ui.toolbox.setItemIcon(2, icon1)

        self.modelMarks = model if model else HashmarksModel()
        self.modelMarks.setHorizontalHeaderLabels([iTitle(), iPath(), iDate()])

        self.ui.treeMarks.doubleClicked.connect(self.onMarkItemDoubleClick)
        self.ui.treeMarks.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ui.treeMarks.customContextMenuRequested.connect(self.onContextMenu)

        self.updatingMarks = False
        self.doMarksUpdate()

    def onMarkDeleted(self, mPath):
        if mPath in self._marksCache:
            self._marksCache.remove(mPath)

    def doMarksUpdate(self):
        self.loop.create_task(self.updateMarks(
            self.modelMarks, self.marksLocal, self.ui.treeMarks))

    def onContextMenu(self, point):
        idx = self.ui.treeMarks.indexAt(point)
        if not idx.isValid():
            return

        idxPath = self.modelMarks.sibling(idx.row(), 1, idx)
        dataPath = self.modelMarks.data(idxPath)
        menu = QMenu()

        if dataPath:
            act2 = menu.addAction(iDelete(), lambda:
                    self.deleteMark(dataPath))

        menu.exec(self.ui.treeMarks.mapToGlobal(point))

    def deleteMark(self, path):
        if self.marksLocal.delete(path):
            modelDelete(self.modelMarks, path)

    def currentItemPath(self):
        idx = self.ui.treeMarks.currentIndex()
        item = self.modelMarks.itemFromIndex(idx)

        if type(item) is CategoryItem:
            return None

        return self.modelMarks.data(
            self.modelMarks.sibling(idx.row(), 1, idx))

    def onDeletePressed(self):
        path = self.currentItemPath()
        if path:
            self.deleteMark(path)

    def onMarkItemChangeShare(self, index, path):
        shareItem = self.modelMarks.itemFromIndex(index)
        markSearch = self.marksLocal.search(path)
        if markSearch:
            path, mark = markSearch
            if mark['share'] is True:
                mark['share'] = False
                shareItem.setText(iNo())
            elif mark['share'] is False:
                mark['share'] = True
                shareItem.setText(iYes())
            self.marksLocal.changed.emit()

    def onMarkItemDoubleClick(self, index):
        indexCat = self.modelMarks.sibling(index.row(), 0, index)
        indexPath = self.modelMarks.sibling(index.row(), 1, index)
        item0 = self.modelMarks.itemFromIndex(indexCat)
        path = self.modelMarks.data(indexPath)

        if type(item0) is CategoryItem:
            return

        tab = self.gWindow.addBrowserTab()
        tab.browseFsPath(path)




import os.path
import uuid
import pkg_resources
import base64
from datetime import datetime
from io import BytesIO

import aiofiles

from PyQt5.QtCore import (pyqtSignal, QObject)

from galacteek import log, ensure
from galacteek.ipfs.mutable import MutableIPFSJson
from galacteek.ipfs.dag import EvolvingDAG
from galacteek.ipfs.wrappers import *
from galacteek.core.asynclib import asyncify, asyncReadFile
from galacteek.web import render
from galacteek.crypto import rsa

class UserInfos(MutableIPFSJson):
    GENDER_MALE = 0
    GENDER_FEMALE = 1
    GENDER_UNSPECIFIED = -1

    def __init__(self, ipfsFilePath, **kw):
        super().__init__(ipfsFilePath)

    def initObj(self):
        uid = str(uuid.uuid4())

        return {
            'userinfo': {
                'username': uid,
                'firstname': '',
                'lastname': '',
                'altname': '',
                'gender': -1,
                'org': '',
                'email': '',
                'country': {
                    'code': '',
                    'name': '',
                },
                'city': '',
                'langs': [],
                'avatar': {
                    'cid': '',
                },
                'crypto': {
                    'rsa': {
                        'pubkeypem': None,
                    },
                    'gpg': {},
                },
                'peerid': '',
                'date': {
                    'created': datetime.now().isoformat(' ', 'seconds'),
                    'modified': datetime.now().isoformat(' ', 'seconds'),
                },
                'uid': uid,
                'locked': False,
                'schemav': 1,
            }
        }

    @property
    def uid(self):
        return self.parser.traverse('userinfo.uid')

    @property
    def peerid(self):
        return self.parser.traverse('userinfo.peerid')

    @property
    def avatarCid(self):
        return self.parser.traverse('userinfo.avatar.cid')

    @property
    def username(self):
        return self.parser.traverse('userinfo.username')

    @property
    def firstname(self):
        return self.parser.traverse('userinfo.firstname')

    @property
    def lastname(self):
        return self.parser.traverse('userinfo.lastname')

    @property
    def email(self):
        return self.parser.traverse('userinfo.email')

    @property
    def org(self):
        return self.parser.traverse('userinfo.org')

    @property
    def countryName(self):
        return self.parser.traverse('userinfo.country.name')

    @property
    def countryCode(self):
        return self.parser.traverse('userinfo.country.code')

    @property
    def locked(self):
        return self.parser.traverse('userinfo.locked')

    @property
    def schemaVersion(self):
        return self.parser.traverse('userinfo.schemav')

    @property
    def objHash(self):
        if self.curEntry:
            return self.curEntry['Hash']

    def setAvatarCid(self, cid):
        if self.locked:
            return
        self.root['userinfo']['avatar']['cid'] = cid
        self.changed.emit()

    def setCountryInfo(self, name, code):
        if self.locked:
            return
        sec = self.root['userinfo']['country']
        sec['name'] = name
        sec['code'] = code
        self.changed.emit()

    def setInfos(self, **kw):
        if self.locked:
            return
        for key, val in kw.items():
            if not key in self.root['userinfo'].keys():
                continue
            self.root['userinfo'][key] = val
        self.root['userinfo']['modified'] = datetime.now().isoformat(
            ' ', 'seconds')
        self.changed.emit()

    def setLock(self, lock=False):
        self.root['userinfo']['locked'] = lock
        self.changed.emit()

    def valid(self):
        return True

class UserDAG(EvolvingDAG):
    def initDag(self):
        return {
            'index.html': 'Blank',
            'media': {
                'images': {},
            },
            'board': {
                'messages': []
            }
        }

class UserApp(QObject):
    def __init__(self, profile):
        super(UserApp, self).__init__(profile)

        self._profile = profile
        self.cssEntry = None

    @property
    def profile(self):
        return self._profile

    @property
    def dagRoot(self):
        return self.profile.dagUser.root

    def debug(self, msg):
        return self.profile.debug(msg)

    @ipfsOp
    async def postMessage(self, op, title, msg):
        async with self.profile.dagUser as dag:
            board = dag.dagRoot['board']
            newMsg = {
                'content': msg,
                'title': title,
                'date': datetime.now().isoformat(' ', 'minutes'),
            }
            board['messages'].append(newMsg)

        await self.update()

    @ipfsOp
    async def init(self, op):
        cssPath = pkg_resources.resource_filename('galacteek.templates',
                'public/css')
        self.cssEntry = await op.addPath(cssPath, recursive=True)

    @ipfsOp
    async def update(self, op):
        homeIndex = await self.profile.tmplRender('public/userhome.html',
                boardMessages=list(reversed(
                    self.dagRoot['board']['messages'])),
                loop=self.profile.ctx.loop)

        async with self.profile.dagUser as dag:
            dag.root['index.html'] = dag.mkLink(homeIndex)
            if self.cssEntry:
                dag.root['css'] = dag.mkLink(self.cssEntry)

            if self.profile.userInfo.avatarCid != '':
                dag.root['media']['images']['avatar'] = dag.mkLink(
                        self.profile.userInfo.avatarCid)

            if self.profile.ctx.hasRsc('ipfs-cube-64'):
                dag.root['media']['images']['ipfs-cube.png'] = dag.mkLink(
                        self.profile.ctx.resources['ipfs-cube-64'])

class UserProfile(QObject):
    """ User profile object """

    P_FILES = 'files'

    def __init__(self, ctx, name, rootDir):
        super(UserProfile, self).__init__()

        self._ctx = ctx
        self._name = name
        self._rootDir = rootDir
        self.keyRoot = 'galacteek.{}.root'.format(self.name)
        self.keyRootId = None

        self._rsaPrivKeyPath = os.path.join(self.ctx.app.cryptoDataLocation,
                'rsa_{0}_priv.key'.format(self.name))
        self._rsaPubKeyPath = os.path.join(self.ctx.app.cryptoDataLocation,
                'rsa_{0}_pub.key'.format(self.name))
        self._rsaPubKey = None

        self._filesModel = None

        self._dagUser = None
        self.userInfo = UserInfos(self.pathUserInfo)
        self.rsa = self.ctx.rsaExec

    def debug(self, msg):
        log.debug('Profile {0}: {1}'.format(self.name, msg))

    @property
    def rsaPrivKeyPath(self):
        return self._rsaPrivKeyPath

    @property
    def rsaPubKeyPath(self):
        return self._rsaPubKeyPath

    @property
    def rsaPubKey(self):
        return self._rsaPubKey

    @rsaPubKey.setter
    def rsaPubKey(self, key):
        self._rsaPubKey = key

    @property
    def dagUser(self):
        return self._dagUser # the DAG for this user

    @property
    def root(self):
        return self._rootDir # profile's root dir in the MFS

    @property
    def ctx(self):
        return self._ctx # IPFS context

    @property
    def name(self):
        return self._name

    @property
    def filesModel(self):
        return self._filesModel

    @property
    def tree(self):
        # Profile's filesystem tree list
        return [
            self.pathFiles,
            self.pathHome,
            self.pathMedia,
            self.pathData,
            self.pathPlaylists,
            self.pathPictures,
            self.pathVideos,
            self.pathDocuments,
            self.pathMusic,
            self.pathCode,
        ]

    @property
    def pathFiles(self):
        return os.path.join(self.root, self.P_FILES)

    @property
    def pathHome(self):
        return os.path.join(self.pathFiles, 'home')

    @property
    def pathDocuments(self):
        return os.path.join(self.pathFiles, 'documents')

    @property
    def pathMedia(self):
        return os.path.join(self.pathFiles, 'multimedia')

    @property
    def pathPictures(self):
        return os.path.join(self.pathMedia, 'pictures')

    @property
    def pathVideos(self):
        return os.path.join(self.pathMedia, 'videos')

    @property
    def pathMusic(self):
        return os.path.join(self.pathMedia, 'music')

    @property
    def pathCode(self):
        return os.path.join(self.pathFiles, 'code')

    @property
    def pathWebsites(self):
        return os.path.join(self.pathFiles, 'websites')

    @property
    def pathEtc(self):
        return os.path.join(self.root, 'etc')

    @property
    def pathData(self):
        return os.path.join(self.root, 'data')

    @property
    def pathPlaylists(self):
        return os.path.join(self.pathData, 'playlists')

    @property
    def pathUserInfo(self):
        return os.path.join(self.pathData, 'userinfo.json')

    @property
    def pathUserDagMeta(self):
        return os.path.join(self.pathData, 'dag.main')

    def setFilesModel(self, model):
        self._filesModel = model

    @ipfsOp
    async def init(self, ipfsop):
        """
        Initialize the profile's filesystem
        """

        await ipfsop.filesMkdir(self.root)

        for directory in self.tree:
            await ipfsop.filesMkdir(directory)

        keysNames = await ipfsop.keysNames()

        if self.keyRoot not in keysNames:
            await ipfsop.keyGen(self.keyRoot)

        key = await ipfsop.keyFind(self.keyRoot)
        if key is not None:
            self.keyRootId = key.get('Id', None)

        self.debug('IPNS key({0}): {1}'.format(self.keyRoot, self.keyRootId))

        ensure(self.userInfo.load())
        await self.userInfo.evLoaded.wait()

        if self.userInfo.avatarCid == '' and self.ctx.hasRsc('ipfs-logo-ice'):
            self.userInfo.setAvatarCid(self.ctx.resources['ipfs-logo-ice']['Hash'])

        if self.userInfo.peerid == '':
            self.userInfo.root['userinfo']['peerid'] = self.ctx.node.id
            self.userInfo.changed.emit()

        if await ipfsop.hasDagCommand():
            self._dagUser = UserDAG(self.pathUserDagMeta, loop=self.ctx.loop)

            ensure(self.dagUser.load())
            await self.dagUser.loaded

            self.dagUser.changed.connect(self.onDagChange)

            self.app = UserApp(self)
            await self.app.init()
            ensure(self.update())
        else:
            self.debug('DAG api not available !')

        self.userInfo.changed.connect(self.onUserInfoChanged)
        ensure(self.initCrypto())

    @ipfsOp
    async def initCrypto(self, op):
        if not os.path.isfile(self.rsaPrivKeyPath) and \
            not os.path.isfile(self.rsaPubKeyPath):

            privKey, pubKey = await self.rsa.genKeys()
            if privKey is None or pubKey is None:
                self.debug('RSA: keygen failed')
                return False

            try:
                async with aiofiles.open(self.rsaPrivKeyPath, 'w+b') as fd:
                    await fd.write(privKey)

                async with aiofiles.open(self.rsaPubKeyPath, 'w+b') as fd:
                    await fd.write(pubKey)
            except Exception as err:
                self.debug('RSA: could not save keys: {}'.format(str(err)))
                return False

        try:
            pubKey = await asyncReadFile(self.rsaPubKeyPath)
            if pubKey is None:
                self.debug('RSA: could not read public key')
                raise Exception('pubkey error')
            self.rsaPubKey = pubKey
        except Exception as e:
            self.debug('RSA: error while importing {}'.format(str(e)))
        else:
            #
            # Tough decision here .. first mechanism tried for publishing the
            # peer's public key payload was to import the PEM key in the repository
            # and transmit the CID, which is neat and handy (other peers just
            # pin/fetch the key..) ATM embedding the key's payload in the
            # userinfo offers some advantages and we can always use another
            # system later on
            #
            rsaSection = self.userInfo.root['userinfo']['crypto'].setdefault(
                'rsa', {})
            rsaSection['pubkeypem'] = self.rsaPubKey.decode()
            return True

    @ipfsOp
    async def rsaEncryptSelf(self, op, data):
        try:
            data = await self.rsa.encryptData(BytesIO(data), self.rsaPubKey)
            entry = await op.client.add_bytes(data)
            if entry:
                self.debug('RSA: self-encrypt: encoded {0} bytes to {1}'.format(
                    len(data), entry['Hash']))
                return entry['Hash']
        except aioipfs.APIError as err:
            self.debug('RSA: IPFS error {}'.format(err.message))

    @ipfsOp
    async def rsaDecryptIpfsObj(self, op, cid):
        privKey = await self.__rsaReadPrivateKey()
        try:
            data = await op.client.cat(cid)
            if data is None:
                raise ValueError('RSA decrypt, IPFS object is invalid')
            decrypted = await self.rsa.decryptData(BytesIO(data), privKey)
            if decrypted:
                self.debug('RSA: decrypted {0} bytes from {1}'.format(
                    len(decrypted), cid))
                return decrypted
        except aioipfs.APIError as err:
            self.debug('RSA: IPFS error {}'.format(err.message))
        except Exception as e:
            self.debug('RSA: unknown error while decrypting {}'.format(str(e)))

    async def __rsaReadPrivateKey(self):
        log.debug('RSA: reading private key')
        return await asyncReadFile(self.rsaPrivKeyPath)

    def onUserInfoChanged(self):
        ensure(self.update())

    def onDagChange(self):
        ensure(self.publishDag())

    @ipfsOp
    async def publishDag(self, op):
        result = await op.publish(self.dagUser.dagCid, key=self.keyRoot)

        if result is None:
            self.debug('DAG publish failed: {}'.format(result))

    async def tmplRender(self, tmpl, **kw):
        """
        Render an HTML template (jinja), passing it the profile, ipfs context
        and the DAG
        """
        return await render.ipfsRender(tmpl,
                profile=self, ipfsCtx=self.ctx, dag=self.dagUser.dagRoot,
                **kw)

    @ipfsOp
    async def update(self, op):
        """
        Update the user's DAG
        """

        await self.dagUser.loaded
        await self.app.update()




import functools

from PyQt5.QtWidgets import QApplication

from galacteek import log
from galacteek.ipfs.ipfsops import *

def _getOp():
    return IPFSOpRegistry.getDefault()

def appTask(fn, *args, **kw):
    app = QApplication.instance()
    app.task(fn, *args, **kw)

def ipfsFunc(func):
    @functools.wraps(func)
    async def wrapper(*args, **kw):
        app = QApplication.instance()
        if not app:
            raise Exception('No Application')
        return await func(app.ipfsClient, *args, **kw)
    return wrapper

def ipfsOpFn(func):
    @functools.wraps(func)
    async def wrapper(*args, **kw):
        op = IPFSOpRegistry.getDefault()
        if op:
            return await func(op, *args, **kw)
        log.debug('ipfsopfn: op is null')
    return wrapper

class ipfsClassW:
    def __init__(self, wrapped):
        self.wrapped = wrapped
        self.name = wrapped.__name__

class ipfsOp(ipfsClassW):
    """
    Wraps an async class method, calling it with an IPFSOperator
    """
    def __get__(self, inst, owner):
        async def wrapper(*args, **kw):
            op = IPFSOpRegistry.getDefault()
            if op:
                return await self.wrapped(inst, op, *args, **kw)
        return wrapper

class ipfsStatOp(ipfsClassW):
    def __get__(self, inst, owner):
        async def wrapper(*args, **kw):
            op = IPFSOpRegistry.getDefault()
            path = args[0]

            stat = op.objStatCtxGet(path)
            if not stat:
                stat = await op.objStatCtxUpdate(path)

            args = args + (stat,)

            return await self.wrapped(inst, op, *args, **kw)
        return wrapper




import os, os.path
import json

defaultConfig = json.loads("""
{
  "API": {
    "HTTPHeaders": null
  },
  "Addresses": {
    "API": "/ip4/127.0.0.1/tcp/5008",
    "Announce": [],
    "Gateway": "/ip4/127.0.0.1/tcp/9001",
    "NoAnnounce": [],
    "Swarm": [
      "/ip4/0.0.0.0/tcp/4008"
    ]
  },
  "Bootstrap": [
    "/dnsaddr/bootstrap.libp2p.io/ipfs/QmNnooDu7bfjPFoTZYxMNLWUQJyrVwtbZg5gBMjTezGAJN",
    "/dnsaddr/bootstrap.libp2p.io/ipfs/QmQCU2EcMqAqQPR2i9bChDtGNJchTbq5TbXJJ16u19uLTa",
    "/dnsaddr/bootstrap.libp2p.io/ipfs/QmbLHAnMoJPWSCR5Zhtx6BHJX9KiKNN6tpvbUcqanj75Nb",
    "/dnsaddr/bootstrap.libp2p.io/ipfs/QmcZf59bWwK5XFi76CZX8cbJ4BhTzzA3gU1ZjYZcYW3dwt",
    "/ip4/104.131.131.82/tcp/4001/ipfs/QmaCpDMGvV2BGHeYERUEnRQAwe3N8SzbUtfsmvsqQLuvuJ",
    "/ip4/104.236.179.241/tcp/4001/ipfs/QmSoLPppuBtQSGwKDZT2M73ULpjvfd3aZ6ha4oFGL1KrGM",
    "/ip4/128.199.219.111/tcp/4001/ipfs/QmSoLSafTMBsPKadTEgaXctDQVcqN88CNLHXMkTNwMKPnu",
    "/ip4/104.236.76.40/tcp/4001/ipfs/QmSoLV4Bbm51jM9C4gDYZQ9Cy3U6aXMJDAbzgu2fzaDs64",
    "/ip4/178.62.158.247/tcp/4001/ipfs/QmSoLer265NRgSp2LA3dPaeykiS1J6DifTC88f5uVQKNAd",
    "/ip6/2604:a880:1:20::203:d001/tcp/4001/ipfs/QmSoLPppuBtQSGwKDZT2M73ULpjvfd3aZ6ha4oFGL1KrGM",
    "/ip6/2400:6180:0:d0::151:6001/tcp/4001/ipfs/QmSoLSafTMBsPKadTEgaXctDQVcqN88CNLHXMkTNwMKPnu",
    "/ip6/2604:a880:800:10::4a:5001/tcp/4001/ipfs/QmSoLV4Bbm51jM9C4gDYZQ9Cy3U6aXMJDAbzgu2fzaDs64",
    "/ip6/2a03:b0c0:0:1010::23:1001/tcp/4001/ipfs/QmSoLer265NRgSp2LA3dPaeykiS1J6DifTC88f5uVQKNAd"
  ],
  "Datastore": {
    "BloomFilterSize": 0,
    "GCPeriod": "1h",
    "HashOnRead": false,
    "Spec": {
      "mounts": [
        {
          "child": {
            "path": "blocks",
            "shardFunc": "/repo/flatfs/shard/v1/next-to-last/2",
            "sync": true,
            "type": "flatfs"
          },
          "mountpoint": "/blocks",
          "prefix": "flatfs.datastore",
          "type": "measure"
        },
        {
          "child": {
            "compression": "none",
            "path": "datastore",
            "type": "levelds"
          },
          "mountpoint": "/",
          "prefix": "leveldb.datastore",
          "type": "measure"
        }
      ],
      "type": "mount"
    },
    "StorageGCWatermark": 90,
    "StorageMax": "10GB"
  },
  "Discovery": {
    "MDNS": {
      "Enabled": true,
      "Interval": 10
    }
  },
  "Experimental": {
    "FilestoreEnabled": false,
    "Libp2pStreamMounting": false,
    "ShardingEnabled": false
  },
  "Gateway": {
    "HTTPHeaders": {
      "Access-Control-Allow-Headers": [
        "X-Requested-With",
        "Range"
      ],
      "Access-Control-Allow-Methods": [
        "GET"
      ],
      "Access-Control-Allow-Origin": [
        "*"
      ]
    },
    "PathPrefixes": [],
    "RootRedirect": "",
    "Writable": false
  },
  "Ipns": {
    "RecordLifetime": "",
    "RepublishPeriod": "",
    "ResolveCacheSize": 128
  },
  "Mounts": {
    "FuseAllowOther": false,
    "IPFS": "/ipfs",
    "IPNS": "/ipns"
  },
  "Reprovider": {
    "Interval": "12h",
    "Strategy": "all"
  },
  "Swarm": {
    "AddrFilters": null,
    "ConnMgr": {
      "GracePeriod": "20s",
      "HighWater": 50,
      "LowWater": 10,
      "Type": "basic"
    },
    "DisableBandwidthMetrics": false,
    "DisableNatPortMap": false,
    "DisableRelay": false,
    "EnableRelayHop": false
  }
}
""")

class Config(object):
    def __init__(self, **kwargs):
        self.c = kwargs.pop('useconfig', defaultConfig)

    def __str__(self):
        return json.dumps(self.c)

def getDefault(*args, **kw):
    return Config(useconfig=defaultConfig)




from galacteek.ipfs.cid import make_cid, CIDv0, CIDv1

import multihash
import re

def isMultihash(hashstring):
    """
    Check if hashstring is a valid base58-encoded multihash

    :param str hashstring: the multihash to validate
    :return: if the value is a valid multihash or not
    :rtype: bool
    """
    try:
        mh = multihash.decode(hashstring.encode('ascii'), 'base58')
        return True
    except:
        return False

def getCID(hashstring):
    try:
        return make_cid(hashstring)
    except:
        return None

def cidValid(cidstring):
    """
    Check if cidstring is a valid IPFS CID

    :param str cidstring: the CID to validate
    :return: if the value is a valid CID or not
    :rtype: bool
    """

    c = getCID(cidstring)
    if c is None:
        return False
    if c.version == 0:
        # Ensure that we can decode the multihash
        try: # can raise ValueError
            if multihash.decode(c.multihash):
                return True
        except:
            return False
    elif c.version == 1:
        return True
    return False

# Regexps

ipfsPathRe = re.compile(
        '^(\s*)?(?:fs:|ipfs:|dweb:)?(?P<fullpath>/ipfs/(?P<cid>[a-zA-Z0-9]{46,49}?)(?P<subpath>\/.*)?)$',
    flags=re.MULTILINE)

ipfsPathGwRe = re.compile(
        '^(\s*)?(?:https?://[a-zA-Z0-9:.]*)?(?P<fullpath>/ipfs/(?P<cid>[a-zA-Z0-9]{46,49}?)(?P<subpath>\/.*)?)$',
    flags=re.MULTILINE)

ipfsCidRe = re.compile(
    '^(\s*)?(?P<cid>[a-zA-Z0-9]{46,49})$', flags=re.MULTILINE)

ipnsPathRe = re.compile(
    '^(\s*)?(?:fs:|ipfs:)?(?P<fullpath>/ipns/([a-zA-Z0-9\.\-]*)(?P<subpath>\/.*)?$)',
    flags=re.MULTILINE)

ipnsPathGwRe = re.compile(
    '^(\s*)?(?:https?://[a-zA-Z0-9:.]*)?(?P<fullpath>/ipns/([a-zA-Z0-9\.\-]*)(?P<subpath>\/.*)?$)',
    flags=re.MULTILINE)

def ipfsRegSearchPath(text):
    for reg in [ ipfsPathRe, ipfsPathGwRe ]:
        matched = reg.match(text)
        if matched:
            return matched
    return None

def ipfsRegSearchCid(text):
    return ipfsCidRe.match(text)

def ipnsRegSearchPath(text):
    for reg in [ ipnsPathRe, ipnsPathGwRe ]:
        matched = reg.match(text)
        if matched:
            return matched
    return None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/hashmarksmgrfeeds.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_FeedsViewForm(object):
    def setupUi(self, FeedsViewForm):
        FeedsViewForm.setObjectName("FeedsViewForm")
        FeedsViewForm.resize(400, 300)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(FeedsViewForm)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.treeFeeds = QtWidgets.QTreeView(FeedsViewForm)
        self.treeFeeds.setObjectName("treeFeeds")
        self.verticalLayout.addWidget(self.treeFeeds)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(FeedsViewForm)
        QtCore.QMetaObject.connectSlotsByName(FeedsViewForm)

    def retranslateUi(self, FeedsViewForm):
        _translate = QtCore.QCoreApplication.translate
        FeedsViewForm.setWindowTitle(_translate("FeedsViewForm", "Form"))




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: unixfs.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='unixfs.proto',
  package='unixfs.pb',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x0cunixfs.proto\x12\tunixfs.pb\"\xdc\x01\n\x04\x44\x61ta\x12&\n\x04Type\x18\x01 \x02(\x0e\x32\x18.unixfs.pb.Data.DataType\x12\x0c\n\x04\x44\x61ta\x18\x02 \x01(\x0c\x12\x10\n\x08\x66ilesize\x18\x03 \x01(\x04\x12\x12\n\nblocksizes\x18\x04 \x03(\x04\x12\x10\n\x08hashType\x18\x05 \x01(\x04\x12\x0e\n\x06\x66\x61nout\x18\x06 \x01(\x04\"V\n\x08\x44\x61taType\x12\x07\n\x03Raw\x10\x00\x12\r\n\tDirectory\x10\x01\x12\x08\n\x04\x46ile\x10\x02\x12\x0c\n\x08Metadata\x10\x03\x12\x0b\n\x07Symlink\x10\x04\x12\r\n\tHAMTShard\x10\x05\"\x1c\n\x08Metadata\x12\x10\n\x08MimeType\x18\x01 \x01(\t')
)



_DATA_DATATYPE = _descriptor.EnumDescriptor(
  name='DataType',
  full_name='unixfs.pb.Data.DataType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Raw', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Directory', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='File', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Metadata', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Symlink', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HAMTShard', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=162,
  serialized_end=248,
)
_sym_db.RegisterEnumDescriptor(_DATA_DATATYPE)


_DATA = _descriptor.Descriptor(
  name='Data',
  full_name='unixfs.pb.Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='Type', full_name='unixfs.pb.Data.Type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Data', full_name='unixfs.pb.Data.Data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filesize', full_name='unixfs.pb.Data.filesize', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blocksizes', full_name='unixfs.pb.Data.blocksizes', index=3,
      number=4, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashType', full_name='unixfs.pb.Data.hashType', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fanout', full_name='unixfs.pb.Data.fanout', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DATA_DATATYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=248,
)


_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='unixfs.pb.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='MimeType', full_name='unixfs.pb.Metadata.MimeType', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=278,
)

_DATA.fields_by_name['Type'].enum_type = _DATA_DATATYPE
_DATA_DATATYPE.containing_type = _DATA
DESCRIPTOR.message_types_by_name['Data'] = _DATA
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Data = _reflection.GeneratedProtocolMessageType('Data', (_message.Message,), dict(
  DESCRIPTOR = _DATA,
  __module__ = 'unixfs_pb2'
  # @@protoc_insertion_point(class_scope:unixfs.pb.Data)
  ))
_sym_db.RegisterMessage(Data)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), dict(
  DESCRIPTOR = _METADATA,
  __module__ = 'unixfs_pb2'
  # @@protoc_insertion_point(class_scope:unixfs.pb.Metadata)
  ))
_sym_db.RegisterMessage(Metadata)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/hashmarksmgr.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_HashmarksViewForm(object):
    def setupUi(self, HashmarksViewForm):
        HashmarksViewForm.setObjectName("HashmarksViewForm")
        HashmarksViewForm.resize(534, 390)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(HashmarksViewForm)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.toolbox = QtWidgets.QToolBox(HashmarksViewForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolbox.sizePolicy().hasHeightForWidth())
        self.toolbox.setSizePolicy(sizePolicy)
        self.toolbox.setObjectName("toolbox")
        self.page = QtWidgets.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 514, 341))
        self.page.setObjectName("page")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.expandButton = QtWidgets.QToolButton(self.page)
        self.expandButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/expand.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.expandButton.setIcon(icon)
        self.expandButton.setCheckable(True)
        self.expandButton.setObjectName("expandButton")
        self.horizontalLayout.addWidget(self.expandButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.treeMarks = QtWidgets.QTreeView(self.page)
        self.treeMarks.setObjectName("treeMarks")
        self.treeMarks.header().setDefaultSectionSize(100)
        self.treeMarks.header().setStretchLastSection(True)
        self.verticalLayout_3.addWidget(self.treeMarks)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.toolbox.addItem(self.page, "")
        self.verticalLayout.addWidget(self.toolbox)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(HashmarksViewForm)
        self.toolbox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(HashmarksViewForm)

    def retranslateUi(self, HashmarksViewForm):
        _translate = QtCore.QCoreApplication.translate
        HashmarksViewForm.setWindowTitle(_translate("HashmarksViewForm", "Form"))
        self.expandButton.setToolTip(_translate("HashmarksViewForm", "<html><head/><body><p>Expand</p></body></html>"))
        self.toolbox.setItemText(self.toolbox.indexOf(self.page), _translate("HashmarksViewForm", "Local hashmarks"))

from . import galacteek_rc




import asyncio
import tempfile
import concurrent.futures
import functools

from io import BytesIO

from Cryptodome.Signature import pss
from Cryptodome.PublicKey import RSA
from Cryptodome.Random import get_random_bytes
from Cryptodome.Cipher import AES, PKCS1_OAEP

from galacteek import log

class RSAExecutor(object):
    """
    RSA Executor.

    The default executor used is a ThreadPoolExecutor
    """
    def __init__(self, loop=None, executor=None):
        self.loop = loop if loop else asyncio.get_event_loop()
        self.executor = executor if executor else \
            concurrent.futures.ThreadPoolExecutor(max_workers=4)

    async def _exec(self, fn, *args, **kw):
        return await self.loop.run_in_executor(self.executor,
                functools.partial(fn, *args, **kw))

    async def importKey(self, keyData):
        return await self._exec(lambda: RSA.import_key(keyData))

    async def genKeys(self, keysize=2048):
        """
        Generate RSA keys of the given keysize (2048 bits by default)
        and return a tuple containing the PEM-encoded private and public key
        """
        def _generateKeypair(size):
            key = RSA.generate(size)
            privKey = key.export_key(pkcs=8, protection='scryptAndAES128-CBC')
            pubKey = key.publickey().export_key()
            return privKey, pubKey

        return await self._exec(_generateKeypair, keysize)

    async def encryptData(self, data, recipientKeyData):
        if not isinstance(data, BytesIO):
            raise ValueError('Need BytesIO')
        return await self._exec(self._encryptPkcs1OAEP, data, recipientKeyData)

    def _encryptPkcs1OAEP(self, data, recipientKeyData):
        try:
            recipientKey = RSA.import_key(recipientKeyData)
            sessionKey = get_random_bytes(16)

            cipherRsa = PKCS1_OAEP.new(recipientKey)
            encSessionKey = cipherRsa.encrypt(sessionKey)

            cipherAes = AES.new(sessionKey, AES.MODE_EAX)
            ciphertext, tag = cipherAes.encrypt_and_digest(data.getvalue())

            fd = BytesIO()
            [ fd.write(x) for x in (encSessionKey,
                cipherAes.nonce, tag, ciphertext) ]
            return fd.getvalue()
        except Exception as err:
            log.debug('RSA encryption error {}'.format(str(err)))
            return None

    async def decryptData(self, data, privKey):
        if not isinstance(data, BytesIO):
            raise ValueError('Need BytesIO')
        return await self._exec(self._decryptPkcs1OAEP, data, privKey)

    def _decryptPkcs1OAEP(self, data, privKey):
        if privKey is None:
            raise ValueError('Invalid private key')

        try:
            private = RSA.import_key(privKey)

            encSessionKey, nonce, tag, ciphertext = [ data.read(x)
                                        for x in (private.size_in_bytes(),
                                        16, 16, -1) ]

            cipherRsa = PKCS1_OAEP.new(private)
            sessionKey = cipherRsa.decrypt(encSessionKey)

            cipherAes = AES.new(sessionKey, AES.MODE_EAX, nonce)
            data = cipherAes.decrypt_and_verify(ciphertext, tag)

            return data
        except ValueError as verr:
            log.debug('RSA decryption error: {}'.format(str(verr)))
            return None
        except TypeError as err:
            log.debug('Type error on decryption, check privkey')
            return None

    async def pssSign(self, message, privRsaKey):
        return await self._exec(self._pssSign, message, privRsaKey)

    def _pssSign(self, message, privRsaKey):
        try:
            key = RSA.import_key(privRsaKey)
            h = SHA256.new(message)
            log.debug('Generaring PSS signature')

            signature = pss.new(key).sign(h)

            out = BytesIO()
            out.write(signature)
            return out.getvalue()
        except Exception as e:
            log.debug('Exception on PSS sign')
            return None

    def _pssVerif(self, message, signature, rsaPubKey):
        try:
            key = RSA.import_key(rsaPubKey)
            hashObj = SHA256.new(message)
            verifier = pss.new(key)
            try:
                verifier.verify(hashObj, signature)
                return True
            except (ValueError, TypeError):
                log.debug('The PSS signature is not authentic')
                return False

        except Exception as e:
            log.debug('Exception on PSS verification')
            return None




import asyncio
import argparse
import sys
import json
import os

from async_generator import async_generator, yield_

import aiohttp

class IPFSSearchResults:
    def __init__(self, page, results):
        self.pageCount = results.get('page_count', 0)
        self.page = page
        self.results = results

    @property
    def hits(self):
        return self.results.get('hits', [])

    def findByHash(self, hashV):
        for hit in self.hits:
            hitHash = hit.get('hash', None)
            if hitHash == hashV:
                return hit

emptyResults = IPFSSearchResults(0, {})

async def searchPage(query, page):
    params = {
            'q': query,
            'page': page
            }

    async with aiohttp.ClientSession() as session:
        async with session.get('https://api.ipfs-search.com/v1/search',
                params=params) as resp:
            return await resp.json()

async def getPageResults(query, page):
    try:
        results = await searchPage(query, page)
        return IPFSSearchResults(page, results)
    except Exception as e:
        return None

@async_generator
async def search(query, preloadPages=0):
    page1Results = await getPageResults(query, 0)
    if page1Results is None:
        await yield_(emptyResults)
        return

    await yield_(page1Results)
    pageCount = page1Results.pageCount

    if preloadPages > 0:
        pageLast = preloadPages if pageCount >= preloadPages else pageCount
        for page in range(page1Results.page+1, pageLast+1):
            results = await getPageResults(query, page)
            if results:
                await yield_(results)




import time
import asyncio
import re
import os, os.path

from PyQt5.QtWidgets import (QWidget, QApplication,
        QDialog, QLabel, QTextEdit, QPushButton, QMessageBox)

from PyQt5.QtCore import QUrl, Qt, pyqtSlot, QCoreApplication
from PyQt5.QtGui import QClipboard, QPixmap, QImage

from galacteek import asyncify, ensure, asyncReadFile
from galacteek.core.ipfsmarks import *
from galacteek.core import countries
from galacteek.ipfs import cidhelpers
from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.wrappers import *

from . import ui_addkeydialog, ui_addhashmarkdialog
from . import ui_addfeeddialog
from . import ui_ipfscidinputdialog, ui_ipfsmultiplecidinputdialog
from . import ui_profileeditdialog
from . import ui_donatedialog
from . import ui_profilepostmessage
from .helpers import *

import mimetypes

def boldLabelStyle():
    return 'QLabel { font-weight: bold; }'

class AddHashmarkDialog(QDialog):
    def __init__(self, marks, resource, title, description, stats, parent=None):
        super().__init__(parent)

        self.ipfsResource = resource
        self.marks = marks
        self.stats = stats if stats else {}

        self.ui = ui_addhashmarkdialog.Ui_AddHashmarkDialog()
        self.ui.setupUi(self)
        self.ui.resourceLabel.setText(self.ipfsResource)
        self.ui.resourceLabel.setStyleSheet(boldLabelStyle())
        self.ui.newCategory.textChanged.connect(self.onNewCatChanged)
        self.ui.title.setText(title)

        if isinstance(description, str):
            self.ui.description.insertPlainText(description)

        for cat in self.marks.getCategories():
            self.ui.category.addItem(cat)

    def onNewCatChanged(self, text):
        if len(text) > 0:
            self.ui.category.setEnabled(False)
        else:
            self.ui.category.setEnabled(True)

    def accept(self):
        share = self.ui.share.isChecked()
        newCat = self.ui.newCategory.text()
        description = self.ui.description.toPlainText()

        if len(newCat) > 0:
            category = newCat
        else:
            category = self.ui.category.currentText()

        mark = IPFSHashMark.make(self.ipfsResource,
            title=self.ui.title.text(),
            share=share,
            comment=self.ui.comment.text(),
            description=description,
            tags=self.ui.tags.text().split(),
            datasize=self.stats.get('DataSize', None),
            cumulativesize=self.stats.get('CumulativeSize', None),
            numlinks=self.stats.get('NumLinks', None)
        )

        self.marks.insertMark(mark, category)
        self.done(0)

class AddFeedDialog(QDialog):
    def __init__(self, marks, resource, feedName=None, parent=None):
        super().__init__(parent)

        self.marks = marks
        self.ipfsResource = resource

        self.ui = ui_addfeeddialog.Ui_AddFeedDialog()
        self.ui.setupUi(self)
        self.ui.resourceLabel.setText(self.ipfsResource)
        self.ui.resourceLabel.setStyleSheet(boldLabelStyle())

        if isinstance(feedName, str):
            self.ui.feedName.setText(feedName)

    def accept(self):
        share = self.ui.share.isChecked()
        autoPin = self.ui.autoPin.isChecked()

        self.marks.follow(self.ipfsResource, self.ui.feedName.text(),
            resolveevery=self.ui.resolve.value(),
            share=share, autoPin=autoPin)
        self.done(0)

class IPFSCIDInputDialog(QDialog):
    """ Dialog for IPFS CID input and validation """

    def __init__(self, parent=None):
        super().__init__(parent)

        self.validCid = False
        self.ui = ui_ipfscidinputdialog.Ui_CIDInputDialog()
        self.ui.setupUi(self)
        self.ui.validity.setStyleSheet(boldLabelStyle())
        self.ui.cid.textChanged.connect(self.onCIDChanged)
        self.ui.clearButton.clicked.connect(self.onClearCID)

    def onClearCID(self):
        self.clearCID()

    def clearCID(self):
        """ Clears the CID input text """
        self.ui.cid.clear()

    def onCIDChanged(self, text):
        """
        When the CID input text changes we verify if it's a valid CID and
        update the validity label
        """
        if cidhelpers.cidValid(text):
            cid = self.getCID()
            if cid:
                self.ui.validity.setText(
                    'Valid CID version {}'.format(cid.version))
                self.validCid = True
            else:
                # Unlikely
                self.ui.validity.setText('Unknown CID type')
                self.validCid = False
        else:
            self.ui.validity.setText('Invalid CID')
            self.validCid = False

    def getHash(self):
        """ Returns the hash corresponding to the input, if valid """
        cid = self.getCID()
        if cid:
            return str(cid)

    def getCID(self):
        """ Returns the CID object corresponding to the input """
        try:
            return cidhelpers.getCID(self.ui.cid.text())
        except:
            return None

    def accept(self):
        if self.validCid is True:
            self.done(1)

class IPFSMultipleCIDInputDialog(QDialog):
    """ Dialog for multiple IPFS CID input """

    def __init__(self, parent=None):
        super().__init__(parent)

        self.validCid = False
        self.ui = ui_ipfsmultiplecidinputdialog.Ui_MultipleCIDInputDialog()
        self.ui.setupUi(self)
        self.ui.validity.setStyleSheet(boldLabelStyle())
        self.ui.cid.textChanged.connect(self.onCIDChanged)
        self.ui.addCidButton.clicked.connect(self.onAddCID)

    def onAddCID(self):
        if self.validCid:
            self.ui.cidList.addItem(self.ui.cid.text())
            self.ui.cid.clear()

    def onCIDChanged(self, text):
        if cidhelpers.cidValid(text):
            self.ui.validity.setText('Valid CID')
            self.validCid = True
        else:
            self.ui.validity.setText('Invalid CID')
            self.validCid = False

    def getCIDs(self):
        cids = []
        for idx in range(0, self.ui.cidList.count()):
            item = self.ui.cidList.item(idx)
            cids.append(item.text())
        return cids

    def accept(self):
        self.done(1)

class DonateDialog(QDialog):
    def __init__(self, bcAddr, parent=None):
        super().__init__(parent)

        self.ui = ui_donatedialog.Ui_DonateDialog()
        self.ui.setupUi(self)
        self.ui.okButton.clicked.connect(self.close)
        self.ui.bitcoinClip.clicked.connect(self.onBcClip)

        self._bcAddr = bcAddr
        self.ui.bitcoinAddress.setText('<b>{0}</b>'.format(self._bcAddr))
        self.setWindowTitle('Make a donation')

    def onBcClip(self):
        self.toClip(self._bcAddr)

    def toClip(self, data):
        app = QCoreApplication.instance()
        app.clipboard().setText(data, QClipboard.Selection)
        app.clipboard().setText(data, QClipboard.Clipboard)

    def close(self):
        self.done(1)

    def accept(self):
        self.done(1)

def notEmpty(v):
    return v != ''

class ProfileEditDialog(QDialog):
    def __init__(self, profile, parent=None):
        super().__init__(parent)

        self.profile = profile
        self.countryList = countries.countryList

        self.ui = ui_profileeditdialog.Ui_ProfileEditDialog()
        self.ui.setupUi(self)

        self.loadCountryData()

        self.ui.labelWarning.setStyleSheet('QLabel { font-weight: bold; }')
        self.ui.username.setText(self.profile.userInfo.username)
        self.ui.firstname.setText(self.profile.userInfo.firstname)
        self.ui.lastname.setText(self.profile.userInfo.lastname)
        self.ui.email.setText(self.profile.userInfo.email)
        self.ui.org.setText(self.profile.userInfo.org)

        if notEmpty(self.profile.userInfo.countryName):
            self.ui.countryBox.setCurrentText(self.profile.userInfo.countryName)
        else:
            self.ui.countryBox.setCurrentText('Unspecified')

        if notEmpty(self.profile.userInfo.avatarCid):
            self.updateAvatarCid()

        self.ui.profileCryptoId.setText('<b>{}</b>'.format(
            self.profile.userInfo.objHash))

        self.ui.changeIconButton.clicked.connect(self.changeIcon)
        self.ui.updateButton.clicked.connect(self.save)
        self.ui.cancelButton.clicked.connect(self.close)

        self.reloadIcon()

    def updateAvatarCid(self):
        self.ui.iconHash.setText('<a href="ipfs:{0}">{1}</a>'.format(
            joinIpfs(self.profile.userInfo.avatarCid),
            self.profile.userInfo.avatarCid))

    def reloadIcon(self):
        ensure(self.loadIcon())

    def getCountryCode(self, name):
        for entry in self.countryList:
            if entry['name'] == name:
                return entry['code']

    def loadCountryData(self):
        for entry in self.countryList:
            self.ui.countryBox.addItem(entry['name'])

        self.ui.countryBox.addItem('Unspecified')

    @asyncify
    async def loadIcon(self):
        @ipfsOpFn
        async def load(op, cid):
            try:
                imgData = await op.client.cat(cid)
                img1 = QImage()
                img1.loadFromData(imgData)
                img = img1.scaledToWidth(256)
                self.ui.iconPixmap.setPixmap(QPixmap.fromImage(img))
            except Exception as e:
                messageBox('Error while loading image')

        if self.profile.userInfo.avatarCid != '':
            await load(self.profile.userInfo.avatarCid)

    def changeIcon(self):
        fps = filesSelect(filter='Images (*.xpm, *.jpg, *.png)')
        if len(fps) > 0:
            ensure(self.setIcon(fps.pop()))

    @ipfsOp
    async def setIcon(self, op, fp):
        entry = await op.addPath(fp, recursive=False)
        if entry:
            self.profile.userInfo.setAvatarCid(entry['Hash'])
            self.reloadIcon()
            self.updateAvatarCid()

    def save(self):
        kw = {}
        for key in ['username', 'firstname',
                    'lastname', 'email', 'org']:
            val = getattr(self.ui, key).text()
            ma = re.search("[a-zA-Z\_\-\@\.\+\'\´0-9\s]*", val)
            if ma:
                kw[key] = val

        country = self.ui.countryBox.currentText()
        code = self.getCountryCode(country)

        if country and code:
            self.profile.userInfo.setCountryInfo(country, code)

        self.profile.userInfo.setInfos(**kw)
        self.done(1)

    def reject(self):
        self.done(0)

class ProfilePostMessageDialog(QDialog):
    def __init__(self, profile, parent=None):
        super().__init__(parent)

        self.profile = profile

        self.ui = ui_profilepostmessage.Ui_PostMessageDialog()
        self.ui.setupUi(self)

    def accept(self):
        ensure(self.post())

    async def post(self):
        msg = self.ui.message.toPlainText()
        title = self.ui.title.text()
        if not title:
            title = 'No title'

        if msg:
            ensure(self.profile.app.postMessage(title, msg))

        self.done(1)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/settings.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsDialogForm(object):
    def setupUi(self, SettingsDialogForm):
        SettingsDialogForm.setObjectName("SettingsDialogForm")
        SettingsDialogForm.resize(724, 478)
        self.verticalLayout = QtWidgets.QVBoxLayout(SettingsDialogForm)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(SettingsDialogForm)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.downloadsLocation = QtWidgets.QLineEdit(self.tab)
        self.downloadsLocation.setObjectName("downloadsLocation")
        self.gridLayout_4.addWidget(self.downloadsLocation, 4, 1, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(280, 18, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tab)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        self.home = QtWidgets.QLineEdit(self.tab)
        self.home.setMinimumSize(QtCore.QSize(400, 0))
        self.home.setObjectName("home")
        self.gridLayout_4.addWidget(self.home, 0, 1, 1, 2)
        self.goToHomePageOnOpen = QtWidgets.QCheckBox(self.tab)
        self.goToHomePageOnOpen.setObjectName("goToHomePageOnOpen")
        self.gridLayout_4.addWidget(self.goToHomePageOnOpen, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(391, 262, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem1, 6, 1, 1, 2)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_4.addWidget(self.label_7, 4, 0, 1, 1)
        self.changeDownloadsPathButton = QtWidgets.QPushButton(self.tab)
        self.changeDownloadsPathButton.setMaximumSize(QtCore.QSize(200, 16777215))
        self.changeDownloadsPathButton.setObjectName("changeDownloadsPathButton")
        self.gridLayout_4.addWidget(self.changeDownloadsPathButton, 5, 1, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_6)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.groupBoxDaemon = QtWidgets.QGroupBox(self.tab_6)
        self.groupBoxDaemon.setMinimumSize(QtCore.QSize(0, 50))
        self.groupBoxDaemon.setCheckable(True)
        self.groupBoxDaemon.setObjectName("groupBoxDaemon")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxDaemon)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_17 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_17.setObjectName("label_17")
        self.gridLayout_5.addWidget(self.label_17, 2, 0, 1, 1)
        self.ipfsdApiPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdApiPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdApiPort.setMinimum(1024)
        self.ipfsdApiPort.setMaximum(32768)
        self.ipfsdApiPort.setObjectName("ipfsdApiPort")
        self.gridLayout_5.addWidget(self.ipfsdApiPort, 1, 1, 1, 1)
        self.ipfsdGwPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdGwPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdGwPort.setMinimum(1024)
        self.ipfsdGwPort.setMaximum(32768)
        self.ipfsdGwPort.setObjectName("ipfsdGwPort")
        self.gridLayout_5.addWidget(self.ipfsdGwPort, 2, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_18.setObjectName("label_18")
        self.gridLayout_5.addWidget(self.label_18, 0, 2, 1, 1)
        self.swarmMinConns = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.swarmMinConns.setMinimum(1)
        self.swarmMinConns.setMaximum(100)
        self.swarmMinConns.setObjectName("swarmMinConns")
        self.gridLayout_5.addWidget(self.swarmMinConns, 0, 3, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_19.setObjectName("label_19")
        self.gridLayout_5.addWidget(self.label_19, 1, 2, 1, 1)
        self.ipfsdSwarmPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdSwarmPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdSwarmPort.setMinimum(1024)
        self.ipfsdSwarmPort.setMaximum(32768)
        self.ipfsdSwarmPort.setObjectName("ipfsdSwarmPort")
        self.gridLayout_5.addWidget(self.ipfsdSwarmPort, 0, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_15.setObjectName("label_15")
        self.gridLayout_5.addWidget(self.label_15, 1, 0, 1, 1)
        self.swarmMaxConns = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.swarmMaxConns.setMinimum(10)
        self.swarmMaxConns.setMaximum(300)
        self.swarmMaxConns.setObjectName("swarmMaxConns")
        self.gridLayout_5.addWidget(self.swarmMaxConns, 1, 3, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_14.setObjectName("label_14")
        self.gridLayout_5.addWidget(self.label_14, 0, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_5.setObjectName("label_5")
        self.gridLayout_5.addWidget(self.label_5, 2, 2, 1, 1)
        self.storageMax = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.storageMax.setMinimum(5)
        self.storageMax.setMaximum(2048)
        self.storageMax.setObjectName("storageMax")
        self.gridLayout_5.addWidget(self.storageMax, 2, 3, 1, 1)
        self.verticalLayout_5.addWidget(self.groupBoxDaemon)
        self.groupBoxIpfsConn = QtWidgets.QGroupBox(self.tab_6)
        self.groupBoxIpfsConn.setEnabled(True)
        self.groupBoxIpfsConn.setMinimumSize(QtCore.QSize(0, 50))
        self.groupBoxIpfsConn.setFlat(False)
        self.groupBoxIpfsConn.setCheckable(False)
        self.groupBoxIpfsConn.setChecked(False)
        self.groupBoxIpfsConn.setObjectName("groupBoxIpfsConn")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBoxIpfsConn)
        self.formLayout_3.setObjectName("formLayout_3")
        self.label_2 = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label_2.setObjectName("label_2")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.label_16 = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label_16.setObjectName("label_16")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_16)
        self.customIpfsApiPort = QtWidgets.QSpinBox(self.groupBoxIpfsConn)
        self.customIpfsApiPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.customIpfsApiPort.setMinimum(1024)
        self.customIpfsApiPort.setMaximum(32768)
        self.customIpfsApiPort.setObjectName("customIpfsApiPort")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.customIpfsApiPort)
        self.label = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label.setObjectName("label")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label)
        self.customIpfsGwPort = QtWidgets.QSpinBox(self.groupBoxIpfsConn)
        self.customIpfsGwPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.customIpfsGwPort.setMinimum(1024)
        self.customIpfsGwPort.setMaximum(32768)
        self.customIpfsGwPort.setObjectName("customIpfsGwPort")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.customIpfsGwPort)
        self.customIpfsHost = QtWidgets.QLineEdit(self.groupBoxIpfsConn)
        self.customIpfsHost.setMaximumSize(QtCore.QSize(300, 16777215))
        self.customIpfsHost.setObjectName("customIpfsHost")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.customIpfsHost)
        self.verticalLayout_5.addWidget(self.groupBoxIpfsConn)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QtWidgets.QLabel(self.tab_6)
        self.label_4.setMaximumSize(QtCore.QSize(721, 145))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.tabWidget.addTab(self.tab_6, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.language = QtWidgets.QComboBox(self.groupBox_2)
        self.language.setObjectName("language")
        self.horizontalLayout.addWidget(self.language)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout.addWidget(self.groupBox_2, 0, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.wrapFiles = QtWidgets.QCheckBox(self.groupBox)
        self.wrapFiles.setGeometry(QtCore.QRect(10, 30, 661, 21))
        self.wrapFiles.setObjectName("wrapFiles")
        self.wrapDirectories = QtWidgets.QCheckBox(self.groupBox)
        self.wrapDirectories.setGeometry(QtCore.QRect(10, 60, 661, 21))
        self.wrapDirectories.setObjectName("wrapDirectories")
        self.hideHashes = QtWidgets.QCheckBox(self.groupBox)
        self.hideHashes.setGeometry(QtCore.QRect(10, 90, 661, 21))
        self.hideHashes.setObjectName("hideHashes")
        self.gridLayout.addWidget(self.groupBox, 1, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_6.addLayout(self.gridLayout_3, 0, 0, 1, 1)
        self.jsIpfsApi = QtWidgets.QCheckBox(self.groupBox_3)
        self.jsIpfsApi.setObjectName("jsIpfsApi")
        self.gridLayout_6.addWidget(self.jsIpfsApi, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox_3, 1, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(SettingsDialogForm)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SettingsDialogForm)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(SettingsDialogForm.accept)
        self.buttonBox.rejected.connect(SettingsDialogForm.reject)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialogForm)

    def retranslateUi(self, SettingsDialogForm):
        _translate = QtCore.QCoreApplication.translate
        SettingsDialogForm.setWindowTitle(_translate("SettingsDialogForm", "Settings"))
        self.label_3.setText(_translate("SettingsDialogForm", "Home:"))
        self.goToHomePageOnOpen.setText(_translate("SettingsDialogForm", "Go to home page when opening a new browser tab"))
        self.label_7.setText(_translate("SettingsDialogForm", "Save downloads to:"))
        self.changeDownloadsPathButton.setText(_translate("SettingsDialogForm", "Change directory"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("SettingsDialogForm", "General"))
        self.groupBoxDaemon.setTitle(_translate("SettingsDialogForm", "Spawn IPFS daemon and use as transport"))
        self.label_17.setText(_translate("SettingsDialogForm", "HTTP gateway port"))
        self.label_18.setText(_translate("SettingsDialogForm", "Minimum swarm connections"))
        self.label_19.setText(_translate("SettingsDialogForm", "Maximum swarm connections"))
        self.label_15.setText(_translate("SettingsDialogForm", "API port"))
        self.label_14.setText(_translate("SettingsDialogForm", "Swarm port"))
        self.label_5.setText(_translate("SettingsDialogForm", "Maximum storage (Gb)"))
        self.groupBoxIpfsConn.setTitle(_translate("SettingsDialogForm", "Use a custom IPFS daemon"))
        self.label_2.setText(_translate("SettingsDialogForm", "Host"))
        self.label_16.setText(_translate("SettingsDialogForm", "API port"))
        self.label.setText(_translate("SettingsDialogForm", "HTTP gateway port"))
        self.label_4.setText(_translate("SettingsDialogForm", "Restart the application after changing the IPFS connection settings."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_6), _translate("SettingsDialogForm", "IPFS"))
        self.groupBox_2.setTitle(_translate("SettingsDialogForm", "Language"))
        self.label_6.setText(_translate("SettingsDialogForm", "Language"))
        self.groupBox.setTitle(_translate("SettingsDialogForm", "File manager"))
        self.wrapFiles.setText(_translate("SettingsDialogForm", "Wrap files with a directory (preserves filenames)"))
        self.wrapDirectories.setText(_translate("SettingsDialogForm", "Wrap directories with a directory"))
        self.hideHashes.setText(_translate("SettingsDialogForm", "Hide IPFS hashes in tree views"))
        self.groupBox_3.setTitle(_translate("SettingsDialogForm", "Browser"))
        self.jsIpfsApi.setText(_translate("SettingsDialogForm", "Enable Javascript IPFS API"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("SettingsDialogForm", "UI"))





import sys
import time
import os.path

from PyQt5.QtWidgets import (QFileDialog, QDialog, QLabel, QAction)
from PyQt5.QtWidgets import QMessageBox

from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import QUrl, Qt, pyqtSlot

from . import ui_settings
from ..appsettings import *
from .helpers import *
from .i18n import *

class SettingsDialog(QDialog):
    def __init__(self, app, parent=None):
        super().__init__(parent)

        self.app = app
        self.sManager = self.app.settingsMgr

        self.ui = ui_settings.Ui_SettingsDialogForm()
        self.ui.setupUi(self)
        self.ui.groupBoxDaemon.toggled.connect(self.onDaemonToggle)
        self.ui.changeDownloadsPathButton.clicked.connect(self.onChangeDownloadsPath)

        self.ui.language.insertItem(0, iLangEnglish())
        self.ui.language.insertItem(1, iLangFrench())

        self.loadSettings()

    def enableGroupDaemon(self):
        self.ui.groupBoxIpfsConn.setEnabled(False)
        self.ui.groupBoxDaemon.setEnabled(True)
        self.ui.groupBoxDaemon.setChecked(True)

    def enableGroupCustom(self):
        self.ui.groupBoxIpfsConn.setEnabled(True)
        self.ui.groupBoxDaemon.setChecked(False)

    def onChangeDownloadsPath(self):
        dirSel = directorySelect()
        if dirSel:
            self.ui.downloadsLocation.setText(dirSel)
            self.setS(CFG_SECTION_BROWSER, CFG_KEY_DLPATH, dirSel)

    def onDaemonToggle(self, on):
        if on:
            self.enableGroupDaemon()
        else:
            self.enableGroupCustom()

    def getS(self, section, key, type=None):
        return self.sManager.getSetting(section, key, type=type)

    def isChecked(self, w):
        return w.checkState() == Qt.Checked

    def setChecked(self, w, bVal):
        if bVal is True:
            w.setCheckState(Qt.Checked)
        else:
            w.setCheckState(Qt.Unchecked)

    def setS(self, section, key, value):
        return self.sManager.setSetting(section, key, value)

    def loadSettings(self):
        # IPFSD
        section = CFG_SECTION_IPFSD
        ipfsdEnabled = self.sManager.isTrue(section, CFG_KEY_ENABLED)

        if ipfsdEnabled:
            self.enableGroupDaemon()
        else:
            self.enableGroupCustom()

        self.ui.ipfsdSwarmPort.setValue(
                self.getS(section, CFG_KEY_SWARMPORT, int))
        self.ui.ipfsdApiPort.setValue(
                self.getS(section, CFG_KEY_APIPORT, int))
        self.ui.ipfsdGwPort.setValue(
                self.getS(section, CFG_KEY_HTTPGWPORT, int))
        self.ui.swarmMinConns.setValue(
                self.getS(section, CFG_KEY_SWARMLOWWATER, int))
        self.ui.swarmMaxConns.setValue(
                self.getS(section, CFG_KEY_SWARMHIGHWATER, int))
        self.ui.storageMax.setValue(
                self.getS(section, CFG_KEY_STORAGEMAX, int))

        # IPFS connection
        section = CFG_SECTION_IPFSCONN1
        self.ui.customIpfsHost.setText(
                self.getS(section, CFG_KEY_HOST, str))
        self.ui.customIpfsApiPort.setValue(
                self.getS(section, CFG_KEY_APIPORT, int))
        self.ui.customIpfsGwPort.setValue(
                self.getS(section, CFG_KEY_HTTPGWPORT, int))

        # Browser
        section = CFG_SECTION_BROWSER
        self.ui.home.setText(
                self.getS(section, CFG_KEY_HOMEURL, str))
        self.ui.downloadsLocation.setText(
                self.getS(section, CFG_KEY_DLPATH, str))
        self.setChecked(self.ui.goToHomePageOnOpen,
                self.sManager.isTrue(section, CFG_KEY_GOTOHOME))
        self.setChecked(self.ui.jsIpfsApi,
                self.sManager.isTrue(section, CFG_KEY_JSAPI))

	# UI
        section = CFG_SECTION_UI
        self.setChecked(self.ui.wrapFiles,
                self.sManager.isTrue(section, CFG_KEY_WRAPSINGLEFILES))
        self.setChecked(self.ui.wrapDirectories,
                self.sManager.isTrue(section, CFG_KEY_WRAPDIRECTORIES))
        self.setChecked(self.ui.hideHashes,
                self.sManager.isTrue(section, CFG_KEY_HIDEHASHES))

        lang = self.sManager.getSetting(section, CFG_KEY_LANG)
        if lang == 'en':
            self.ui.language.setCurrentText(iLangEnglish())
        elif lang == 'fr':
            self.ui.language.setCurrentText(iLangFrench())

    def accept(self):
        section = CFG_SECTION_IPFSD

        if self.ui.groupBoxDaemon.isChecked():
            self.sManager.setTrue(section, CFG_KEY_ENABLED)
        else:
            self.sManager.setFalse(section, CFG_KEY_ENABLED)

        self.setS(section, CFG_KEY_SWARMPORT, self.ui.ipfsdSwarmPort.text())
        self.setS(section, CFG_KEY_APIPORT, self.ui.ipfsdApiPort.text())
        self.setS(section, CFG_KEY_HTTPGWPORT, self.ui.ipfsdGwPort.text())
        self.setS(section, CFG_KEY_SWARMLOWWATER, self.ui.swarmMinConns.text())
        self.setS(section, CFG_KEY_SWARMHIGHWATER, self.ui.swarmMaxConns.text())
        self.setS(section, CFG_KEY_STORAGEMAX, self.ui.storageMax.text())

        section = CFG_SECTION_IPFSCONN1
        self.setS(section, CFG_KEY_HOST, self.ui.customIpfsHost.text())
        self.setS(section, CFG_KEY_APIPORT, self.ui.customIpfsApiPort.text())
        self.setS(section, CFG_KEY_HTTPGWPORT, self.ui.customIpfsGwPort.text())

        section = CFG_SECTION_BROWSER
        self.setS(section, CFG_KEY_HOMEURL, self.ui.home.text())
        self.sManager.setBoolFrom(section, CFG_KEY_GOTOHOME,
                self.isChecked(self.ui.goToHomePageOnOpen))
        self.sManager.setBoolFrom(section, CFG_KEY_JSAPI,
                self.isChecked(self.ui.jsIpfsApi))

        section = CFG_SECTION_UI
        self.sManager.setBoolFrom(section, CFG_KEY_WRAPSINGLEFILES,
                self.isChecked(self.ui.wrapFiles))
        self.sManager.setBoolFrom(section, CFG_KEY_WRAPDIRECTORIES,
                self.isChecked(self.ui.wrapDirectories))
        self.sManager.setBoolFrom(section, CFG_KEY_HIDEHASHES,
                self.isChecked(self.ui.hideHashes))

        lang = self.ui.language.currentText()
        if lang == iLangEnglish():
            self.sManager.setSetting(section, CFG_KEY_LANG, 'en')
        elif lang == iLangFrench():
            self.sManager.setSetting(section, CFG_KEY_LANG, 'fr')
        self.app.setupTranslator()

        self.sManager.sync()
        self.done(1)

    def reject(self):
        self.done(0)




import asyncio
import time
import os.path
import base64, base58

from async_generator import async_generator, yield_, yield_from_

from PyQt5.QtCore import (pyqtSignal, QObject)

from galacteek import log, ensure
from galacteek.ipfs.wrappers import ipfsOp
from galacteek.ipfs.ipfsops import *
from galacteek.ipfs import pb
from galacteek.core.asynclib import asyncify, async_enterable
from galacteek.core.jtraverse import traverseParser

class DAGObj:
    def __init__(self, data):
        self.data = data

class DAGQuery(object):
    def __init__(self, dagCid=None, dagRoot=None):
        self._dagCid = dagCid
        self._dagRoot = dagRoot
        self.evLoaded = asyncio.Event()

    def debug(self, msg):
        log.debug('DAG query: {}'.format(msg))

    @property
    def dagCid(self):
        return self._dagCid

    @property
    def dagRoot(self):
        return self._dagRoot

    def path(self, subpath):
        return os.path.join(joinIpfs(self.dagCid), subpath)

    async def waitLoaded(self, timeout=15):
        return asyncio.wait_for(self.evLoaded.wait(), timeout)

    @ipfsOp
    async def load(self, op, timeout=10):
        self._dagRoot = await op.waitFor(op.dagGet(self.dagCid), timeout)
        if self.dagRoot:
            self.evLoaded.set()
            return self.dagRoot

    @ipfsOp
    async def get(self, op, path):
        dagNode = await op.dagGet(
            os.path.join(self.dagCid, path))

        if isinstance(dagNode, dict) and 'data' in dagNode and \
                'links' in dagNode:
            """ Try and decode the protobuf data """
            msg = pb.decodeDagNode(dagNode['data'])
            if not msg:
                return
            return msg['data']
        else:
            return dagNode

    @ipfsOp
    async def cat(self, op, path):
        return await op.client.cat(
            os.path.join(self.dagCid, path))

    @ipfsOp
    async def list(self, op, path=''):
        data = await self.get(path)
        if data is None:
            return

        if isinstance(data, dict):
            return data.keys()

    @ipfsOp
    async def resolve(self, op, path=''):
        return await op.dagResolve(
            os.path.join(self.dagCid, path))

    @async_generator
    async def walk(self, op, path='', maxObjSize=0, depth=1):
        """
        Do not use this!
        """

        data = await self.get(path)
        if data is None:
            return

        if isinstance(data, bytes):
            """ Protobuf data decoded by get() """
            await yield_((path, DAGObj(data)))

        elif isinstance(data, dict):
            for objKey, objValue  in data.items():
                if objKey == '/':
                    out = await op.client.cat(objValue)
                    await yield_((path, DAGObj(out)))
                if objKey == 'data' or objKey == 'links':
                    continue
                else:
                    await yield_from_(
                            self.walk(op, path=os.path.join(path, objKey),
                                maxObjSize=maxObjSize, depth=depth)
                        )

        elif isinstance(data, list):
            for idx, obj in enumerate(data):
                await yield_from_(
                        self.walk(op, path=os.path.join(path, str(idx)),
                            maxObjSize=maxObjSize)
                        )
        elif isinstance(data, str):
            await yield_((path, DAGObj(data)))

    async def __aenter__(self):
        if self.dagRoot is None:
            await self.load()
            if self.dagRoot is None:
                raise Exception('Cannot load dag')
        return self

    async def __aexit__(self, *args):
        pass

class EvolvingDAG(QObject):
    """
    :param str dagMetaMfsPath: the path inside the MFS for the metadata
        describing this DAG
    """

    changed = pyqtSignal()
    metadataEntryChanged = pyqtSignal()
    available = pyqtSignal(object)

    keyCidLatest = 'cidlatest'

    def __init__(self, dagMetaMfsPath, dagMetaHistoryMax=12, loop=None):
        super().__init__()

        self.lock = asyncio.Lock()
        self.loop = loop if loop else asyncio.get_event_loop()
        self.loaded = asyncio.Future()

        self._curMetaEntry = None
        self._dagRoot = None
        self._dagMeta = None
        self._dagCid = None
        self._dagMetaMaxHistoryItems = dagMetaHistoryMax
        self._dagMetaMfsPath = dagMetaMfsPath

    @property
    def dagMetaMfsPath(self):
        return self._dagMetaMfsPath # path inside the mfs

    @property
    def dagMetaMaxHistoryItems(self):
        return self._dagMetaMaxHistoryItems

    @property
    def curMetaEntry(self):
        return self._curMetaEntry

    @curMetaEntry.setter
    def curMetaEntry(self, value):
        self._curMetaEntry = value

    @property
    def dagCid(self):
        return self._dagCid

    @dagCid.setter
    def dagCid(self, cid):
        self._dagMeta[self.keyCidLatest] = cid
        self._dagCid = cid
        self.debug("DAG's CID now is {0}".format(cid))

    @property
    def dagMeta(self):
        return self._dagMeta

    @property
    def dagRoot(self):
        return self._dagRoot

    @property
    def root(self):
        return self._dagRoot

    def debug(self, msg):
        log.debug('EDAG ({meta}): {msg}'.format(
            meta=self.dagMetaMfsPath, msg=msg))

    def initDag(self):
        return {}

    def mkLink(self, cid):
        if isinstance(cid, str):
            return {"/": cid}
        elif isinstance(cid, dict):
            # assume it's a dict which has a 'Hash' key
            return {"/": cid['Hash']}

    async def load(self):
        await self.loadDag()

    @ipfsOp
    async def loadDag(self, op):
        meta = await op.filesReadJsonObject(self.dagMetaMfsPath)

        if meta is not None:
            self._dagMeta = meta
            latest = self.dagMeta.get(self.keyCidLatest, None)
            if latest:
                self.dagCid = latest
                self._dagRoot = await op.dagGet(self.dagCid)
            else:
                # How inconvenient ..
                # TODO: the history could be used here to recreate a DAG
                # from previous CIDs but we really shouldn't have to enter here
                self._dagRoot = {}

            self.loaded.set_result(True)
        else:
            self._dagMeta = {
                self.keyCidLatest: None,
                'history': []
            }
            self._dagRoot = self.initDag()
            await self.ipfsSave()
            self.loaded.set_result(True)

        self.parser = traverseParser(self.dagRoot)
        self.loop.call_soon(self.available.emit, self.dagRoot)

    @ipfsOp
    async def ipfsSave(self, op):
        with await self.lock:
            prevCid = self.dagCid
            history = self.dagMeta.setdefault('history', [])
            maxItems = self.dagMetaMaxHistoryItems

            # We always PIN the latest DAG and do a pin update using the
            # previous item in the history

            cid = await op.dagPut(self.dagRoot, pin=True)
            if cid is not None:
                if prevCid is not None and prevCid not in history:
                    if len(history) > maxItems:
                        # Purge old items
                        [history.pop() for idx in range(0,
                            len(history) - maxItems) ]

                    history.insert(0, prevCid)
                    await op.pinUpdate(prevCid, cid)

                # Save the new CID and update the metadata
                self.dagCid = cid
                resp = await op.filesWriteJsonObject(self.dagMetaMfsPath,
                        self.dagMeta)
                entry = await op.filesStat(self.dagMetaMfsPath)
                if entry:
                    self.curMetaEntry = entry
            else:
                # Bummer
                self.debug('DAG could not be built')
                return False

        return True

    @ipfsOp
    async def get(self, op, path):
        self.debug('DAG get {}'.format(os.path.join(self.dagCid, path)))
        dagNode = await op.dagGet(
            os.path.join(self.dagCid, path))

        if isinstance(dagNode, dict) and 'data' in dagNode and \
                'links' in dagNode:
            """ Try and decode the protobuf data """
            msg = pb.decodeDagNode(dagNode['data'])
            if not msg:
                return
            return msg['data']
        else:
            return dagNode

    @ipfsOp
    async def cat(self, op, path):
        return await op.client.cat(
            os.path.join(self.dagCid, path))

    @ipfsOp
    async def list(self, op, path=''):
        data = await self.get(path)
        if data is None:
            return

        if isinstance(data, dict):
            return data.keys()

    @ipfsOp
    async def resolve(self, op, path=''):
        return await op.dagResolve(
            os.path.join(self.dagCid, path))

    @async_generator
    async def walk(self, op, path='', maxObjSize=0, depth=1):
        """
        Do not use this!
        """

        data = await self.get(path)
        if data is None:
            return

        if isinstance(data, dict) and 'data' in data and 'links' in data:
            """ Try and decode the protobuf data """
            msg = pb.decodeDagNode(data['data'])
            if not msg:
                return
            if maxObjSize == 0 or (maxObjSize > 0 and msg['size'] <
                    maxObjSize):
                await yield_((path, DAGObj(msg['data'])))

        if not data:
            return

        if isinstance(data, dict):
            for objKey, objValue  in data.items():
                if objKey == '/':
                    out = await op.client.cat(objValue)
                    await yield_((path, DAGObj(out)))
                if objKey == 'data' or objKey == 'links':
                    continue
                else:
                    await yield_from_(
                            self.walk(op, path=os.path.join(path, objKey),
                                maxObjSize=maxObjSize, depth=depth)
                        )

        elif isinstance(data, list):
            for idx, obj in enumerate(data):
                await yield_from_(
                        self.walk(op, path=os.path.join(path, str(idx)),
                            maxObjSize=maxObjSize)
                        )
        elif isinstance(data, str):
            await yield_((path, DAGObj(data)))

    async def __aenter__(self):
        """
        Lock is acquired on entering the async ctx manager
        """
        await self.lock.acquire()
        return self

    async def __aexit__(self, *args):
        """
        Release the lock and save. The changed signal is emitted
        """
        self.lock.release()
        await self.ipfsSave()
        self.loop.call_soon(self.changed.emit)

    @async_enterable
    async def query(self):
        return DAGQuery(dagCid=self.dagCid, dagRoot=self.dagRoot)
